@@ -2,14 +2,12 @@
 set -e
 set -x
 
-#get any missing deps from the container
-./contrib/ci/generate_dependencies.py | xargs dnf install -y
-
 #generate a tarball
 git config tar.tar.xz.command "xz -c"
 mkdir -p build && pushd build
 rm -rf *
 meson .. \
+    -Db_sanitize=address \
     -Dgtkdoc=true \
     -Dman=true \
     -Dtests=true \
@@ -22,12 +20,13 @@ meson .. \
     -Dplugin_synaptics=true $@
 ninja-build dist
 popd
-VERSION=`meson introspect build --projectinfo | jq -r .version`
+VERSION=`./contrib/get-version.py`
 mkdir -p $HOME/rpmbuild/SOURCES/
 mv build/meson-dist/fwupd-$VERSION.tar.xz $HOME/rpmbuild/SOURCES/
 
 #generate a spec file
 sed "s,#VERSION#,$VERSION,;
+     s,#TARBALL_VERSION#,$VERSION,;
      s,#BUILD#,1,;
      s,#LONGDATE#,`date '+%a %b %d %Y'`,;
      s,#ALPHATAG#,alpha,;
@@ -55,7 +54,7 @@ mkdir -p dist
 cp $HOME/rpmbuild/RPMS/*/*.rpm dist
 
 if [ "$CI" = "true" ]; then
-	sed "s,^BlacklistPlugins=test,BlacklistPlugins=," -i /etc/fwupd/daemon.conf
+	sed "s,^BlacklistPlugins=test;invalid,BlacklistPlugins=," -i /etc/fwupd/daemon.conf
 
 	# set up enough PolicyKit and D-Bus to run the daemon
 	mkdir -p /run/dbus
