@@ -61,7 +61,8 @@ if not os.path.exists(input):
 with open(input, 'r') as rfd:
     lines = rfd.readlines()
 
-with open('Dockerfile', 'w') as wfd:
+out = tempfile.NamedTemporaryFile(dir='.', delete=True)
+with open(out.name, 'w') as wfd:
     for line in lines:
         if line.startswith("FROM %%%ARCH_PREFIX%%%"):
             if (OS == "debian" or OS == "ubuntu") and SUBOS == "i386":
@@ -78,7 +79,7 @@ with open('Dockerfile', 'w') as wfd:
                 wfd.write("RUN apt update -qq && \\\n")
                 wfd.write("\tapt install -yq --no-install-recommends\\\n")
             elif OS == "arch":
-                wfd.write("RUN pacman -Syu --noconfirm \\\n")
+                wfd.write("RUN pacman -Syu --noconfirm --needed\\\n")
             for i in range(0, len(deps)):
                 if i < len(deps)-1:
                     wfd.write("\t%s \\\n" % deps[i])
@@ -100,5 +101,5 @@ with open('Dockerfile', 'w') as wfd:
         args += ['--build-arg=http_proxy=%s' % os.environ['http_proxy']]
     if 'https_proxy' in os.environ:
         args += ['--build-arg=https_proxy=%s' % os.environ['https_proxy']]
-    args += ["-f", "./Dockerfile", "."]
+    args += [ "-f", "./%s" % os.path.basename(out.name), "."]
     subprocess.check_call(args)
