@@ -44,10 +44,12 @@ typedef struct {
 	gchar				*vendor_id;
 	gchar				*homepage;
 	gchar				*plugin;
+	gchar				*protocol;
 	gchar				*version;
 	gchar				*version_lowest;
 	gchar				*version_bootloader;
 	FwupdVersionFormat		 version_format;
+	guint64				 version_raw;
 	GPtrArray			*checksums;
 	guint32				 flashes_left;
 	guint32				 install_duration;
@@ -61,6 +63,8 @@ typedef struct {
 enum {
 	PROP_0,
 	PROP_VERSION_FORMAT,
+	PROP_FLAGS,
+	PROP_PROTOCOL,
 	PROP_LAST
 };
 
@@ -843,6 +847,42 @@ fwupd_device_set_plugin (FwupdDevice *device, const gchar *plugin)
 	priv->plugin = g_strdup (plugin);
 }
 
+/**
+ * fwupd_device_get_protocol:
+ * @device: A #FwupdDevice
+ *
+ * Gets the protocol that the device uses for updating.
+ *
+ * Returns: the protocol name, or %NULL if unset
+ *
+ * Since: 1.3.6
+ **/
+const gchar *
+fwupd_device_get_protocol (FwupdDevice *device)
+{
+	FwupdDevicePrivate *priv = GET_PRIVATE (device);
+	g_return_val_if_fail (FWUPD_IS_DEVICE (device), NULL);
+	return priv->protocol;
+}
+
+/**
+ * fwupd_device_set_protocol:
+ * @device: A #FwupdDevice
+ * @protocol: the protocol name, e.g. `com.hughski.colorhug`
+ *
+ * Sets the protocol that is used to update the device.
+ *
+ * Since: 1.3.6
+ **/
+void
+fwupd_device_set_protocol (FwupdDevice *device, const gchar *protocol)
+{
+	FwupdDevicePrivate *priv = GET_PRIVATE (device);
+	g_return_if_fail (FWUPD_IS_DEVICE (device));
+	g_free (priv->protocol);
+	priv->protocol = g_strdup (protocol);
+}
+
 /**
  * fwupd_device_get_flags:
  * @device: A #FwupdDevice
@@ -875,7 +915,10 @@ fwupd_device_set_flags (FwupdDevice *device, guint64 flags)
 {
 	FwupdDevicePrivate *priv = GET_PRIVATE (device);
 	g_return_if_fail (FWUPD_IS_DEVICE (device));
+	if (priv->flags == flags)
+		return;
 	priv->flags = flags;
+	g_object_notify (G_OBJECT (device), "flags");
 }
 
 /**
@@ -892,7 +935,12 @@ fwupd_device_add_flag (FwupdDevice *device, FwupdDeviceFlags flag)
 {
 	FwupdDevicePrivate *priv = GET_PRIVATE (device);
 	g_return_if_fail (FWUPD_IS_DEVICE (device));
+	if (flag == 0)
+		return;
+	if ((priv->flags & flag) > 0)
+		return;
 	priv->flags |= flag;
+	g_object_notify (G_OBJECT (device), "flags");
 }
 
 /**
@@ -909,7 +957,12 @@ fwupd_device_remove_flag (FwupdDevice *device, FwupdDeviceFlags flag)
 {
 	FwupdDevicePrivate *priv = GET_PRIVATE (device);
 	g_return_if_fail (FWUPD_IS_DEVICE (device));
+	if (flag == 0)
+		return;
+	if ((priv->flags & flag) == 0)
+		return;
 	priv->flags &= ~flag;
+	g_object_notify (G_OBJECT (device), "flags");
 }
 
 /**
@@ -1047,6 +1100,8 @@ fwupd_device_incorporate (FwupdDevice *self, FwupdDevice *donor)
 		fwupd_device_set_vendor_id (self, priv_donor->vendor_id);
 	if (priv->plugin == NULL)
 		fwupd_device_set_plugin (self, priv_donor->plugin);
+	if (priv->protocol == NULL)
+		fwupd_device_set_protocol (self, priv_donor->protocol);
 	if (priv->update_error == NULL)
 		fwupd_device_set_update_error (self, priv_donor->update_error);
 	if (priv->update_message == NULL)
@@ -1059,6 +1114,8 @@ fwupd_device_incorporate (FwupdDevice *self, FwupdDevice *donor)
 		fwupd_device_set_version_bootloader (self, priv_donor->version_bootloader);
 	if (priv->version_format == FWUPD_VERSION_FORMAT_UNKNOWN)
 		fwupd_device_set_version_format (self, priv_donor->version_format);
+	if (priv->version_raw == 0)
+		fwupd_device_set_version_raw (self, priv_donor->version_raw);
 	for (guint i = 0; i < priv_donor->guids->len; i++) {
 		const gchar *tmp = g_ptr_array_index (priv_donor->guids, i);
 		fwupd_device_add_guid (self, tmp);
@@ -1183,6 +1240,11 @@ fwupd_device_to_variant_full (FwupdDevice *device, FwupdDeviceFlags flags)
 				       FWUPD_RESULT_KEY_PLUGIN,
 				       g_variant_new_string (priv->plugin));
 	}
+	if (priv->protocol != NULL) {
+		g_variant_builder_add (&builder, "{sv}",
+				       FWUPD_RESULT_KEY_PROTOCOL,
+				       g_variant_new_string (priv->protocol));
+	}
 	if (priv->version != NULL) {
 		g_variant_builder_add (&builder, "{sv}",
 				       FWUPD_RESULT_KEY_VERSION,
@@ -1198,6 +1260,11 @@ fwupd_device_to_variant_full (FwupdDevice *device, FwupdDeviceFlags flags)
 				       FWUPD_RESULT_KEY_VERSION_BOOTLOADER,
 				       g_variant_new_string (priv->version_bootloader));
 	}
+	if (priv->version_raw > 0) {
+		g_variant_builder_add (&builder, "{sv}",
+				       FWUPD_RESULT_KEY_VERSION_RAW,
+				       g_variant_new_uint64 (priv->version_raw));
+	}
 	if (priv->flashes_left > 0) {
 		g_variant_builder_add (&builder, "{sv}",
 				       FWUPD_RESULT_KEY_FLASHES_LEFT,
@@ -1365,6 +1432,10 @@ fwupd_device_from_key_value (FwupdDevice *device, const gchar *key, GVariant *va
 		fwupd_device_set_plugin (device, g_variant_get_string (value, NULL));
 		return;
 	}
+	if (g_strcmp0 (key, FWUPD_RESULT_KEY_PROTOCOL) == 0) {
+		fwupd_device_set_protocol (device, g_variant_get_string (value, NULL));
+		return;
+	}
 	if (g_strcmp0 (key, FWUPD_RESULT_KEY_VERSION) == 0) {
 		fwupd_device_set_version (device, g_variant_get_string (value, NULL));
 		return;
@@ -1401,6 +1472,10 @@ fwupd_device_from_key_value (FwupdDevice *device, const gchar *key, GVariant *va
 		fwupd_device_set_version_format (device, g_variant_get_uint32 (value));
 		return;
 	}
+	if (g_strcmp0 (key, FWUPD_RESULT_KEY_VERSION_RAW) == 0) {
+		fwupd_device_set_version_raw (device, g_variant_get_uint64 (value));
+		return;
+	}
 }
 
 static void
@@ -1530,6 +1605,41 @@ fwupd_device_set_version_format (FwupdDevice *device, FwupdVersionFormat version
 	priv->version_format = version_format;
 }
 
+/**
+ * fwupd_device_get_version_raw:
+ * @device: A #FwupdDevice
+ *
+ * Gets the raw version number from the hardware before converted to a string.
+ *
+ * Returns: the hardware version, or 0 if unset
+ *
+ * Since: 1.3.6
+ **/
+guint64
+fwupd_device_get_version_raw (FwupdDevice *device)
+{
+	FwupdDevicePrivate *priv = GET_PRIVATE (device);
+	g_return_val_if_fail (FWUPD_IS_DEVICE (device), 0);
+	return priv->version_raw;
+}
+
+/**
+ * fwupd_device_set_version_raw:
+ * @device: A #FwupdDevice
+ * @version_raw: the raw hardware version
+ *
+ * Sets the raw version number from the hardware before converted to a string.
+ *
+ * Since: 1.3.6
+ **/
+void
+fwupd_device_set_version_raw (FwupdDevice *device, guint64 version_raw)
+{
+	FwupdDevicePrivate *priv = GET_PRIVATE (device);
+	g_return_if_fail (FWUPD_IS_DEVICE (device));
+	priv->version_raw = version_raw;
+}
+
 /**
  * fwupd_device_get_update_message:
  * @device: A #FwupdDevice
@@ -1717,6 +1827,7 @@ fwupd_device_to_json (FwupdDevice *device, JsonBuilder *builder)
 	fwupd_device_json_add_string (builder, FWUPD_RESULT_KEY_SUMMARY, priv->summary);
 	fwupd_device_json_add_string (builder, FWUPD_RESULT_KEY_DESCRIPTION, priv->description);
 	fwupd_device_json_add_string (builder, FWUPD_RESULT_KEY_PLUGIN, priv->plugin);
+	fwupd_device_json_add_string (builder, FWUPD_RESULT_KEY_PROTOCOL, priv->protocol);
 	if (priv->flags != FWUPD_DEVICE_FLAG_NONE) {
 		json_builder_set_member_name (builder, FWUPD_RESULT_KEY_FLAGS);
 		json_builder_begin_array (builder);
@@ -1746,6 +1857,8 @@ fwupd_device_to_json (FwupdDevice *device, JsonBuilder *builder)
 	fwupd_device_json_add_string (builder, FWUPD_RESULT_KEY_VERSION_FORMAT,
 				      fwupd_version_format_to_string (priv->version_format));
 	fwupd_device_json_add_int (builder, FWUPD_RESULT_KEY_FLASHES_LEFT, priv->flashes_left);
+	if (priv->version_raw > 0)
+		fwupd_device_json_add_int (builder, FWUPD_RESULT_KEY_VERSION_RAW, priv->version_raw);
 	if (priv->icons->len > 0) {
 		json_builder_set_member_name (builder, "Icons");
 		json_builder_begin_array (builder);
@@ -1789,7 +1902,6 @@ fwupd_device_to_string (FwupdDevice *device)
 {
 	FwupdDevicePrivate *priv = GET_PRIVATE (device);
 	GString *str;
-	g_autoptr(GHashTable) ids = NULL;
 
 	g_return_val_if_fail (FWUPD_IS_DEVICE (device), NULL);
 
@@ -1800,28 +1912,37 @@ fwupd_device_to_string (FwupdDevice *device)
 		str = g_string_append (str, "Unknown Device\n");
 	fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_DEVICE_ID, priv->id);
 	fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_PARENT_DEVICE_ID, priv->parent_id);
-	ids = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
-	for (guint i = 0; i < priv->instance_ids->len; i++) {
-		const gchar *instance_id = g_ptr_array_index (priv->instance_ids, i);
-		g_hash_table_insert (ids,
-				     fwupd_guid_hash_string (instance_id),
-				     g_strdup (instance_id));
-	}
-	for (guint i = 0; i < priv->guids->len; i++) {
-		const gchar *guid = g_ptr_array_index (priv->guids, i);
-		const gchar *instance_id = g_hash_table_lookup (ids, guid);
-		if (instance_id == NULL) {
-			fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_GUID, guid);
-		} else {
-			g_autofree gchar *tmp = NULL;
-			tmp = g_strdup_printf ("%s <- %s", guid, instance_id);
-			fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_GUID, tmp);
+	if (priv->guids->len > 0) {
+		g_autoptr(GHashTable) ids = NULL;
+		ids = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+		for (guint i = 0; i < priv->instance_ids->len; i++) {
+			const gchar *instance_id = g_ptr_array_index (priv->instance_ids, i);
+			g_hash_table_insert (ids,
+					     fwupd_guid_hash_string (instance_id),
+					     g_strdup (instance_id));
+		}
+		for (guint i = 0; i < priv->guids->len; i++) {
+			const gchar *guid = g_ptr_array_index (priv->guids, i);
+			const gchar *instance_id = g_hash_table_lookup (ids, guid);
+			if (instance_id == NULL) {
+				fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_GUID, guid);
+			} else {
+				g_autofree gchar *tmp = NULL;
+				tmp = g_strdup_printf ("%s <- %s", guid, instance_id);
+				fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_GUID, tmp);
+			}
+		}
+	} else {
+		for (guint i = 0; i < priv->instance_ids->len; i++) {
+			const gchar *instance_id = g_ptr_array_index (priv->instance_ids, i);
+			fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_INSTANCE_IDS, instance_id);
 		}
 	}
 	fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_SERIAL, priv->serial);
 	fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_SUMMARY, priv->summary);
 	fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_DESCRIPTION, priv->description);
 	fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_PLUGIN, priv->plugin);
+	fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_PROTOCOL, priv->protocol);
 	fwupd_pad_kv_dfl (str, FWUPD_RESULT_KEY_FLAGS, priv->flags);
 	for (guint i = 0; i < priv->checksums->len; i++) {
 		const gchar *checksum = g_ptr_array_index (priv->checksums, i);
@@ -1837,6 +1958,17 @@ fwupd_device_to_string (FwupdDevice *device)
 			  fwupd_version_format_to_string (priv->version_format));
 	if (priv->flashes_left < 2)
 		fwupd_pad_kv_int (str, FWUPD_RESULT_KEY_FLASHES_LEFT, priv->flashes_left);
+	if (priv->version_raw > 0) {
+		g_autofree gchar *tmp = NULL;
+		if (priv->version_raw > 0xffffffff) {
+			tmp = g_strdup_printf ("0x%08x%08x",
+					       (guint) (priv->version_raw >> 32),
+					       (guint) (priv->version_raw & 0xffffffff));
+		} else {
+			tmp = g_strdup_printf ("0x%08x", (guint) priv->version_raw);
+		}
+		fwupd_pad_kv_str (str, FWUPD_RESULT_KEY_VERSION_RAW, tmp);
+	}
 	if (priv->icons->len > 0) {
 		g_autoptr(GString) tmp = g_string_new (NULL);
 		for (guint i = 0; i < priv->icons->len; i++) {
@@ -1873,6 +2005,12 @@ fwupd_device_get_property (GObject *object, guint prop_id,
 	case PROP_VERSION_FORMAT:
 		g_value_set_uint (value, priv->version_format);
 		break;
+	case PROP_FLAGS:
+		g_value_set_uint64 (value, priv->flags);
+		break;
+	case PROP_PROTOCOL:
+		g_value_set_string (value, priv->protocol);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1888,6 +2026,12 @@ fwupd_device_set_property (GObject *object, guint prop_id,
 	case PROP_VERSION_FORMAT:
 		fwupd_device_set_version_format (self, g_value_get_uint (value));
 		break;
+	case PROP_FLAGS:
+		fwupd_device_set_flags (self, g_value_get_uint64 (value));
+		break;
+	case PROP_PROTOCOL:
+		fwupd_device_set_protocol (self, g_value_get_string (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1911,6 +2055,19 @@ fwupd_device_class_init (FwupdDeviceClass *klass)
 				   G_PARAM_READWRITE |
 				   G_PARAM_STATIC_NAME);
 	g_object_class_install_property (object_class, PROP_VERSION_FORMAT, pspec);
+
+	pspec = g_param_spec_uint64 ("flags", NULL, NULL,
+				     FWUPD_DEVICE_FLAG_NONE,
+				     FWUPD_DEVICE_FLAG_UNKNOWN,
+				     FWUPD_DEVICE_FLAG_NONE,
+				     G_PARAM_READWRITE |
+				     G_PARAM_STATIC_NAME);
+	g_object_class_install_property (object_class, PROP_FLAGS, pspec);
+
+	pspec = g_param_spec_string ("protocol", NULL, NULL, NULL,
+				     G_PARAM_READWRITE |
+				     G_PARAM_STATIC_NAME);
+	g_object_class_install_property (object_class, PROP_PROTOCOL, pspec);
 }
 
 static void
@@ -1941,6 +2098,7 @@ fwupd_device_finalize (GObject *object)
 	g_free (priv->vendor);
 	g_free (priv->vendor_id);
 	g_free (priv->plugin);
+	g_free (priv->protocol);
 	g_free (priv->update_error);
 	g_free (priv->update_message);
 	g_free (priv->version);
