@@ -67,6 +67,8 @@ fwupd_error_to_string (FwupdError error)
 		return FWUPD_DBUS_INTERFACE ".BrokenSystem";
 	if (error == FWUPD_ERROR_BATTERY_LEVEL_TOO_LOW)
 		return FWUPD_DBUS_INTERFACE ".BatteryLevelTooLow";
+	if (error == FWUPD_ERROR_NEEDS_USER_ACTION)
+		return FWUPD_DBUS_INTERFACE ".NeedsUserAction";
 	return NULL;
 }
 
@@ -115,6 +117,8 @@ fwupd_error_from_string (const gchar *error)
 		return FWUPD_ERROR_BROKEN_SYSTEM;
 	if (g_strcmp0 (error, FWUPD_DBUS_INTERFACE ".BatteryLevelTooLow") == 0)
 		return FWUPD_ERROR_BATTERY_LEVEL_TOO_LOW;
+	if (g_strcmp0 (error, FWUPD_DBUS_INTERFACE ".NeedsUserAction") == 0)
+		return FWUPD_ERROR_NEEDS_USER_ACTION;
 	return FWUPD_ERROR_LAST;
 }
 
