@@ -136,6 +136,7 @@ fu_dell_dock_tbt_write_fw (FuDevice *device,
 	}
 
 	/* dock will reboot to re-read; this is to appease the daemon */
+	fu_device_set_status (device, FWUPD_STATUS_DEVICE_RESTART);
 	fu_device_set_version (device, dynamic_version, FWUPD_VERSION_FORMAT_PAIR);
 
 	return TRUE;
@@ -223,6 +224,8 @@ fu_dell_dock_tbt_probe (FuDevice *device, GError **error)
 	fu_device_set_physical_id (device, fu_device_get_physical_id (parent));
 	fu_device_set_logical_id (FU_DEVICE (device), "tbt");
 	fu_device_add_instance_id (device, DELL_DOCK_TBT_INSTANCE_ID);
+	/* this is true only when connected to non-thunderbolt port */
+	fu_device_add_flag (device, FWUPD_DEVICE_FLAG_USABLE_DURING_UPDATE);
 
 	return TRUE;
 }
@@ -277,8 +280,10 @@ fu_dell_dock_tbt_finalize (GObject *object)
 }
 
 static void
-fu_dell_dock_tbt_init (FuDellDockTbt *device)
-{}
+fu_dell_dock_tbt_init (FuDellDockTbt *self)
+{
+	fu_device_set_protocol (FU_DEVICE (self), "com.intel.thunderbolt");
+}
 
 static void
 fu_dell_dock_tbt_class_init (FuDellDockTbtClass *klass)
