@@ -8,8 +8,11 @@
 
 #include <string.h>
 
+#include "fu-chunk.h"
+
 #include "fu-ebitdo-common.h"
 #include "fu-ebitdo-device.h"
+#include "fu-ebitdo-firmware.h"
 
 struct _FuEbitdoDevice {
 	FuUsbDevice		 parent_instance;
@@ -225,6 +228,7 @@ fu_ebitdo_device_set_version (FuEbitdoDevice *self, guint32 version)
 	g_autofree gchar *tmp = NULL;
 	tmp = g_strdup_printf ("%u.%02u", version / 100, version % 100);
 	fu_device_set_version (FU_DEVICE (self), tmp, FWUPD_VERSION_FORMAT_PAIR);
+	fu_device_set_version_raw (FU_DEVICE (self), version);
 }
 
 static gboolean
@@ -348,12 +352,6 @@ fu_ebitdo_device_setup (FuDevice *device, GError **error)
 	return TRUE;
 }
 
-const guint32 *
-fu_ebitdo_device_get_serial (FuEbitdoDevice *self)
-{
-	return self->serial;
-}
-
 static gboolean
 fu_ebitdo_device_write_firmware (FuDevice *device,
 				 FuFirmware *firmware,
@@ -361,13 +359,14 @@ fu_ebitdo_device_write_firmware (FuDevice *device,
 				 GError **error)
 {
 	FuEbitdoDevice *self = FU_EBITDO_DEVICE (device);
-	FuEbitdoFirmwareHeader *hdr;
-	const guint8 *payload_data;
-	const guint chunk_sz = 32;
-	guint32 payload_len;
+	GUsbDevice *usb_device = fu_usb_device_get_dev (FU_USB_DEVICE (device));
+	const guint8 *buf;
+	gsize bufsz = 0;
 	guint32 serial_new[3];
-	g_autoptr(GBytes) fw = NULL;
+	g_autoptr(GBytes) fw_hdr = NULL;
+	g_autoptr(GBytes) fw_payload = NULL;
 	g_autoptr(GError) error_local = NULL;
+	g_autoptr(GPtrArray) chunks = NULL;
 	const guint32 app_key_index[16] = {
 		0x186976e5, 0xcac67acd, 0x38f27fee, 0x0a4948f1,
 		0xb75b7753, 0x1f8ffa5c, 0xbff8cf43, 0xc4936167,
@@ -377,7 +376,6 @@ fu_ebitdo_device_write_firmware (FuDevice *device,
 
 	/* not in bootloader mode, so print what to do */
 	if (fu_device_has_flag (device, FWUPD_DEVICE_FLAG_NEEDS_BOOTLOADER)) {
-		GUsbDevice *usb_device = fu_usb_device_get_dev (FU_USB_DEVICE (device));
 		g_autoptr(GString) msg = g_string_new ("Not in bootloader mode: ");
 		g_string_append (msg, "Disconnect the controller, ");
 		g_print ("1. \n");
@@ -400,6 +398,7 @@ fu_ebitdo_device_write_firmware (FuDevice *device,
 			break;
 		case 0x6000: /* SF30 pro: Dinput mode */
 		case 0x6001: /* SN30 pro: Dinput mode */
+		case 0x6002: /* SN30 pro+: Dinput mode */
 		case 0x028e: /* SF30/SN30 pro: Xinput mode */
 		case 0x5006: /* M30 */
 			g_string_append (msg, "press and hold L1+R1+START for 3 seconds "
@@ -412,112 +411,67 @@ fu_ebitdo_device_write_firmware (FuDevice *device,
 		g_string_append (msg, "then re-connect controller");
 		g_set_error_literal (error,
 				     FWUPD_ERROR,
-				     FWUPD_ERROR_NOT_SUPPORTED,
+				     FWUPD_ERROR_NEEDS_USER_ACTION,
 				     msg->str);
 		return FALSE;
 	}
 
-	/* get default image */
-	fw = fu_firmware_get_image_default_bytes (firmware, error);
-	if (fw == NULL)
+	/* get header and payload */
+	fw_hdr = fu_firmware_get_image_by_id_bytes (firmware,
+						    FU_FIRMWARE_IMAGE_ID_HEADER,
+						    error);
+	if (fw_hdr == NULL)
 		return FALSE;
-
-	/* corrupt */
-	if (g_bytes_get_size (fw) < sizeof (FuEbitdoFirmwareHeader)) {
-		g_set_error_literal (error,
-				     G_IO_ERROR,
-				     G_IO_ERROR_INVALID_DATA,
-				     "firmware too small for header");
+	fw_payload = fu_firmware_get_image_by_id_bytes (firmware,
+							FU_FIRMWARE_IMAGE_ID_PAYLOAD,
+							error);
+	if (fw_payload == NULL)
 		return FALSE;
-	}
-
-	/* print details about the firmware */
-	hdr = (FuEbitdoFirmwareHeader *) g_bytes_get_data (fw, NULL);
-	fu_ebitdo_dump_firmware_header (hdr);
-
-	/* check the file size */
-	payload_len = (guint32) (g_bytes_get_size (fw) - sizeof (FuEbitdoFirmwareHeader));
-	if (payload_len != GUINT32_FROM_LE (hdr->destination_len)) {
-		g_set_error (error,
-			     G_IO_ERROR,
-			     G_IO_ERROR_INVALID_DATA,
-			     "file size incorrect, expected 0x%04x got 0x%04x",
-			     (guint) GUINT32_FROM_LE (hdr->destination_len),
-			     (guint) payload_len);
-		return FALSE;
-	}
-
-	/* check if this is firmware */
-	for (guint i = 0; i < 4; i++) {
-		if (hdr->reserved[i] != 0x0) {
-			g_set_error (error,
-				     G_IO_ERROR,
-				     G_IO_ERROR_INVALID_DATA,
-				     "data invalid, reserved[%u] = 0x%04x",
-				     i, hdr->reserved[i]);
-			return FALSE;
-		}
-	}
 
 	/* set up the firmware header */
 	fu_device_set_status (device, FWUPD_STATUS_DEVICE_WRITE);
+	buf = g_bytes_get_data (fw_hdr, &bufsz);
 	if (!fu_ebitdo_device_send (self,
-				 FU_EBITDO_PKT_TYPE_USER_CMD,
-				 FU_EBITDO_PKT_CMD_UPDATE_FIRMWARE_DATA,
-				 FU_EBITDO_PKT_CMD_FW_UPDATE_HEADER,
-				 (const guint8 *) hdr, sizeof(FuEbitdoFirmwareHeader),
-				 &error_local)) {
-		g_set_error (error,
-			     G_IO_ERROR,
-			     G_IO_ERROR_INVALID_DATA,
-			     "failed to set up firmware header: %s",
-			     error_local->message);
+				    FU_EBITDO_PKT_TYPE_USER_CMD,
+				    FU_EBITDO_PKT_CMD_UPDATE_FIRMWARE_DATA,
+				    FU_EBITDO_PKT_CMD_FW_UPDATE_HEADER,
+				    buf, bufsz, error)) {
+		g_prefix_error (error, "failed to set up firmware header:");
 		return FALSE;
 	}
-	if (!fu_ebitdo_device_receive (self, NULL, 0, &error_local)) {
-		g_set_error (error,
-			     G_IO_ERROR,
-			     G_IO_ERROR_INVALID_DATA,
-			     "failed to get ACK for fw update header: %s",
-			     error_local->message);
+	if (!fu_ebitdo_device_receive (self, NULL, 0, error)) {
+		g_prefix_error (error, "failed to get ACK for fw update header: ");
 		return FALSE;
 	}
 
 	/* flash the firmware in 32 byte blocks */
-	payload_data = g_bytes_get_data (fw, NULL);
-	payload_data += sizeof(FuEbitdoFirmwareHeader);
-	for (guint32 offset = 0; offset < payload_len; offset += chunk_sz) {
+	chunks = fu_chunk_array_new_from_bytes (fw_payload, 0x0, 0x0, 32);
+	for (guint i = 0; i < chunks->len; i++) {
+		FuChunk *chunk = g_ptr_array_index (chunks, i);
 		if (g_getenv ("FWUPD_EBITDO_VERBOSE") != NULL) {
 			g_debug ("writing %u bytes to 0x%04x of 0x%04x",
-				 chunk_sz, offset, payload_len);
+				 chunk->data_sz, chunk->address, chunk->data_sz);
 		}
-		fu_device_set_progress_full (device, offset, payload_len);
 		if (!fu_ebitdo_device_send (self,
-					 FU_EBITDO_PKT_TYPE_USER_CMD,
-					 FU_EBITDO_PKT_CMD_UPDATE_FIRMWARE_DATA,
-					 FU_EBITDO_PKT_CMD_FW_UPDATE_DATA,
-					 payload_data + offset, chunk_sz,
-					 &error_local)) {
-			g_set_error (error,
-				     G_IO_ERROR,
-				     G_IO_ERROR_INVALID_DATA,
-				     "Failed to write firmware @0x%04x: %s",
-				     offset, error_local->message);
+					    FU_EBITDO_PKT_TYPE_USER_CMD,
+					    FU_EBITDO_PKT_CMD_UPDATE_FIRMWARE_DATA,
+					    FU_EBITDO_PKT_CMD_FW_UPDATE_DATA,
+					    chunk->data, chunk->data_sz,
+					    error)) {
+			g_prefix_error (error,
+					"failed to write firmware @0x%04x: ",
+					chunk->address);
 			return FALSE;
 		}
-		if (!fu_ebitdo_device_receive (self, NULL, 0, &error_local)) {
-			g_set_error (error,
-				     G_IO_ERROR,
-				     G_IO_ERROR_INVALID_DATA,
-				     "Failed to get ACK for write firmware @0x%04x: %s",
-				     offset, error_local->message);
+		if (!fu_ebitdo_device_receive (self, NULL, 0, error)) {
+			g_prefix_error (error,
+					"failed to get ACK for write firmware @0x%04x: ",
+					chunk->address);
 			return FALSE;
 		}
+		fu_device_set_progress_full (device, chunk->idx, chunks->len);
 	}
 
-	/* mark as complete */
-	fu_device_set_progress_full (device, payload_len, payload_len);
-
 	/* set the "encode id" which is likely a checksum, bluetooth pairing
 	 * or maybe just security-through-obscurity -- also note:
 	 * SET_ENCODE_ID enforces no read for success?! */
@@ -530,12 +484,8 @@ fu_ebitdo_device_write_firmware (FuDevice *device,
 				 FU_EBITDO_PKT_CMD_FW_SET_ENCODE_ID,
 				 (guint8 *) serial_new,
 				 sizeof(serial_new),
-				 &error_local)) {
-		g_set_error (error,
-			     G_IO_ERROR,
-			     G_IO_ERROR_INVALID_DATA,
-			     "failed to set encoding ID: %s",
-			     error_local->message);
+				 error)) {
+		g_prefix_error (error, "failed to set encoding ID: ");
 		return FALSE;
 	}
 
@@ -545,23 +495,41 @@ fu_ebitdo_device_write_firmware (FuDevice *device,
 				 FU_EBITDO_PKT_CMD_UPDATE_FIRMWARE_DATA,
 				 FU_EBITDO_PKT_CMD_FW_UPDATE_OK,
 				 NULL, 0,
-				 &error_local)) {
-		g_set_error (error,
-			     G_IO_ERROR,
-			     G_IO_ERROR_INVALID_DATA,
-			     "failed to mark firmware as successful: %s",
-			     error_local->message);
+				 error)) {
+		g_prefix_error (error, "failed to mark firmware as successful: ");
 		return FALSE;
 	}
 	if (!fu_ebitdo_device_receive (self, NULL, 0, &error_local)) {
-		g_set_error (error,
-			     G_IO_ERROR,
-			     G_IO_ERROR_INVALID_DATA,
-			     "failed to get ACK for mark firmware as successful: %s",
-			     error_local->message);
+		g_prefix_error (&error_local, "failed to get ACK for mark firmware as successful: ");
+		if (fu_device_has_flag (device, FWUPD_DEVICE_FLAG_WILL_DISAPPEAR)) {
+			fu_device_set_remove_delay (device, 0);
+			g_debug ("%s", error_local->message);
+			return TRUE;
+		}
+		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
 
+	/* when doing a soft-reboot the device does not re-enumerate properly
+	 * so manually reboot the GUsbDevice */
+	fu_device_set_status (device, FWUPD_STATUS_DEVICE_RESTART);
+	if (!g_usb_device_reset (usb_device, &error_local)) {
+		g_prefix_error (&error_local, "failed to force-reset device: ");
+		if (fu_device_has_flag (device, FWUPD_DEVICE_FLAG_WILL_DISAPPEAR)) {
+			fu_device_set_remove_delay (device, 0);
+			g_debug ("%s", error_local->message);
+			return TRUE;
+		}
+		g_propagate_error (error, g_steal_pointer (&error_local));
+		return FALSE;
+	}
+
+	/* not all 8bito devices come back in the right mode */
+	if (fu_device_has_flag (device, FWUPD_DEVICE_FLAG_WILL_DISAPPEAR))
+		fu_device_set_remove_delay (device, 0);
+	else
+		fu_device_add_flag (device, FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG);
+
 	/* success! */
 	return TRUE;
 }
@@ -595,9 +563,23 @@ fu_ebitdo_device_probe (FuUsbDevice *device, GError **error)
 	return TRUE;
 }
 
+static FuFirmware *
+fu_ebitdo_device_prepare_firmware (FuDevice *device,
+				   GBytes *fw,
+				   FwupdInstallFlags flags,
+				   GError **error)
+{
+	g_autoptr(FuFirmware) firmware = fu_ebitdo_firmware_new ();
+	fu_device_set_status (device, FWUPD_STATUS_DECOMPRESSING);
+	if (!fu_firmware_parse (firmware, fw, flags, error))
+		return NULL;
+	return g_steal_pointer (&firmware);
+}
+
 static void
 fu_ebitdo_device_init (FuEbitdoDevice *self)
 {
+	fu_device_set_protocol (FU_DEVICE (self), "com.8bitdo");
 }
 
 static void
@@ -609,22 +591,5 @@ fu_ebitdo_device_class_init (FuEbitdoDeviceClass *klass)
 	klass_device->setup = fu_ebitdo_device_setup;
 	klass_usb_device->open = fu_ebitdo_device_open;
 	klass_usb_device->probe = fu_ebitdo_device_probe;
-}
-
-/**
- * fu_ebitdo_device_new:
- *
- * Creates a new #FuEbitdoDevice.
- *
- * Returns: (transfer full): a #FuEbitdoDevice, or %NULL if not a game pad
- *
- * Since: 0.1.0
- **/
-FuEbitdoDevice *
-fu_ebitdo_device_new (FuUsbDevice *device)
-{
-	FuEbitdoDevice *self;
-	self = g_object_new (FU_TYPE_EBITDO_DEVICE, NULL);
-	fu_device_incorporate (FU_DEVICE (self), FU_DEVICE (device));
-	return FU_EBITDO_DEVICE (self);
+	klass_device->prepare_firmware = fu_ebitdo_device_prepare_firmware;
 }
