@@ -7,6 +7,7 @@
 #include "config.h"
 
 #include "fu-plugin-vfuncs.h"
+#include "fu-hash.h"
 
 struct FuPluginData {
 	GMutex			 mutex;
@@ -15,11 +16,7 @@ struct FuPluginData {
 void
 fu_plugin_init (FuPlugin *plugin)
 {
-	if (g_strcmp0 (g_getenv ("FWUPD_PLUGIN_TEST"), "build-hash") == 0)
-		fu_plugin_set_build_hash (plugin, "invalid");
-	else
-		fu_plugin_set_build_hash (plugin, FU_BUILD_HASH);
-	fu_plugin_add_rule (plugin, FU_PLUGIN_RULE_SUPPORTS_PROTOCOL, "com.acme.test");
+	fu_plugin_set_build_hash (plugin, FU_BUILD_HASH);
 	fu_plugin_alloc_data (plugin, sizeof (FuPluginData));
 	g_debug ("init");
 }
@@ -41,6 +38,8 @@ fu_plugin_coldplug (FuPlugin *plugin, GError **error)
 	fu_device_set_name (device, "Integrated_Webcam(TM)");
 	fu_device_add_icon (device, "preferences-desktop-keyboard");
 	fu_device_add_flag (device, FWUPD_DEVICE_FLAG_UPDATABLE);
+	fu_device_add_flag (device, FWUPD_DEVICE_FLAG_CAN_VERIFY);
+	fu_device_set_protocol (device, "com.acme.test");
 	fu_device_set_summary (device, "A fake webcam");
 	fu_device_set_vendor (device, "ACME Corp.");
 	fu_device_set_vendor_id (device, "USB:0x046D");
@@ -100,6 +99,11 @@ fu_plugin_verify (FuPlugin *plugin,
 		  FuPluginVerifyFlags flags,
 		  GError **error)
 {
+	if (g_strcmp0 (fu_device_get_version (device), "1.2.2") == 0) {
+		fu_device_add_checksum (device, "90d0ad436d21e0687998cd2127b2411135e1f730");
+		fu_device_add_checksum (device, "921631916a60b295605dbae6a0309f9b64e2401b3de8e8506e109fc82c586e3a");
+		return TRUE;
+	}
 	if (g_strcmp0 (fu_device_get_version (device), "1.2.3") == 0) {
 		fu_device_add_checksum (device, "7998cd212721e068b2411135e1f90d0ad436d730");
 		fu_device_add_checksum (device, "dbae6a0309b3de8e850921631916a60b2956056e109fc82c586e3f9b64e2401a");
