@@ -13,12 +13,15 @@
 
 #include "fu-device-metadata.h"
 
+#include "fu-common.h"
+
 #include "fu-uefi-common.h"
 #include "fu-uefi-device.h"
 #include "fu-uefi-devpath.h"
 #include "fu-uefi-bootmgr.h"
 #include "fu-uefi-pcrs.h"
 #include "fu-uefi-vars.h"
+#include "fu-uefi-udisks.h"
 
 struct _FuUefiDevice {
 	FuDevice		 parent_instance;
@@ -31,6 +34,7 @@ struct _FuUefiDevice {
 	guint32			 last_attempt_version;
 	guint64			 fmp_hardware_instance;
 	gboolean		 missing_header;
+	gboolean		 automounted_esp;
 };
 
 G_DEFINE_TYPE (FuUefiDevice, fu_uefi_device, FU_TYPE_DEVICE)
@@ -105,6 +109,10 @@ fu_uefi_device_to_string (FuDevice *device, guint idt, GString *str)
 	fu_common_string_append_kx (str, idt, "LastAttemptVersion", self->last_attempt_version);
 	fu_common_string_append_kv (str, idt, "EspPath",
 				    fu_device_get_metadata (device, "EspPath"));
+	fu_common_string_append_ku (str, idt, "RequireESPFreeSpace",
+				    fu_device_get_metadata_integer (device, "RequireESPFreeSpace"));
+	fu_common_string_append_kb (str, idt, "RequireShimForSecureBoot",
+				    fu_device_get_metadata_boolean (device, "RequireShimForSecureBoot"));
 }
 
 FuUefiDeviceKind
@@ -363,7 +371,7 @@ fu_uefi_device_write_update_info (FuUefiDevice *self,
 	};
 
 	/* set the body as the device path */
-	if (g_getenv ("FWUPD_UEFI_ESP_PATH") != NULL) {
+	if (g_getenv ("FWUPD_UEFI_TEST") != NULL) {
 		g_debug ("not building device path, in tests....");
 		return TRUE;
 	}
@@ -371,7 +379,7 @@ fu_uefi_device_write_update_info (FuUefiDevice *self,
 	/* convert to EFI device path */
 	dp_buf = fu_uefi_device_build_dp_buf (filename, &dp_bufsz, error);
 	if (dp_buf == NULL) {
-		fu_uefi_prefix_efi_errors (error);
+		fu_uefi_print_efivar_errors ();
 		return FALSE;
 	}
 
@@ -387,12 +395,156 @@ fu_uefi_device_write_update_info (FuUefiDevice *self,
 				    FU_UEFI_VARS_ATTR_BOOTSERVICE_ACCESS |
 				    FU_UEFI_VARS_ATTR_RUNTIME_ACCESS,
 				    error)) {
-		fu_uefi_prefix_efi_errors (error);
+		fu_uefi_print_efivar_errors ();
 		return FALSE;
 	}
 	return TRUE;
 }
 
+static gboolean
+fu_uefi_device_is_esp_mounted (FuDevice *device, GError **error)
+{
+	const gchar *esp_path = fu_device_get_metadata (device, "EspPath");
+	g_autofree gchar *contents = NULL;
+	g_auto(GStrv) lines = NULL;
+	gsize length;
+
+	if (esp_path == NULL) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOT_SUPPORTED,
+				     "EFI System partition is not defined");
+		return FALSE;
+	}
+
+	if (!g_file_get_contents ("/proc/mounts", &contents, &length, error))
+		return FALSE;
+	lines = g_strsplit (contents, "\n", 0);
+
+	for (guint i = 0; lines[i] != NULL; i++) {
+		if (lines[i] != NULL && g_strrstr (lines[i], esp_path))
+			return TRUE;
+	}
+	g_set_error (error,
+		     FWUPD_ERROR,
+		     FWUPD_ERROR_NOT_SUPPORTED,
+		     "EFI System partition %s is not mounted",
+		     esp_path);
+	return FALSE;
+}
+
+static gboolean
+fu_uefi_device_check_esp_free (FuDevice *device, GError **error)
+{
+	const gchar *esp_path = fu_device_get_metadata (device, "EspPath");
+	guint64 sz_reqd = fu_device_get_metadata_integer (device, "RequireESPFreeSpace");
+	if (sz_reqd == G_MAXUINT) {
+		g_debug ("maximum size is not configured");
+		return TRUE;
+	}
+	return 	fu_uefi_check_esp_free_space (esp_path, sz_reqd, error);
+}
+
+static gboolean
+fu_uefi_device_cleanup_esp (FuDevice *device, GError **error)
+{
+	const gchar *esp_path = fu_device_get_metadata (device, "EspPath");
+	g_autofree gchar *pattern = NULL;
+	g_autoptr(GPtrArray) files = NULL;
+
+	/* in case we call capsule install twice before reboot */
+	if (fu_uefi_vars_exists (FU_UEFI_VARS_GUID_EFI_GLOBAL, "BootNext"))
+		return TRUE;
+
+	/* delete any files matching the glob in the ESP */
+	files = fu_common_get_files_recursive (esp_path, error);
+	if (files == NULL)
+		return FALSE;
+	pattern = g_build_filename (esp_path, "EFI/*/fw/fwupd-*.cap", NULL);
+	for (guint i = 0; i < files->len; i++) {
+		const gchar *fn = g_ptr_array_index (files, i);
+		if (fu_common_fnmatch (pattern, fn)) {
+			g_autoptr(GFile) file = g_file_new_for_path (fn);
+			g_debug ("deleting %s", fn);
+			if (!g_file_delete (file, NULL, error))
+				return FALSE;
+		}
+	}
+
+	/* delete any old variables */
+	if (!fu_uefi_vars_delete_with_glob (FU_UEFI_VARS_GUID_FWUPDATE, "fwupd-*", error))
+		return FALSE;
+
+	return TRUE;
+}
+
+static gboolean
+fu_uefi_device_prepare (FuDevice *device,
+			FwupdInstallFlags flags,
+			GError **error)
+{
+	/* not set in conf, figure it out */
+	if (fu_device_get_metadata (device, "EspPath") == NULL) {
+		g_autofree gchar *guessed = NULL;
+		g_autofree gchar *detected_esp = NULL;
+		guessed = fu_uefi_guess_esp_path (error);
+		if (guessed == NULL)
+			return FALSE;
+
+		/* udisks objpath */
+		if (fu_uefi_udisks_objpath (guessed)) {
+			FuUefiDevice *self = FU_UEFI_DEVICE (device);
+			detected_esp = fu_uefi_udisks_objpath_is_mounted (guessed);
+			if (detected_esp != NULL) {
+				g_debug ("ESP already mounted @ %s", detected_esp);
+			/* not mounted */
+			} else {
+				g_debug ("Mounting ESP @ %s", guessed);
+				detected_esp = fu_uefi_udisks_objpath_mount (guessed, error);
+				if (detected_esp == NULL)
+					return FALSE;
+				self->automounted_esp = TRUE;
+			}
+		/* already mounted */
+		} else {
+			detected_esp = g_steal_pointer (&guessed);
+		}
+		fu_device_set_metadata (device, "EspPath", detected_esp);
+	}
+
+	/* sanity checks */
+	if (!fu_uefi_device_is_esp_mounted (device, error))
+		return FALSE;
+	if (!fu_uefi_device_check_esp_free (device, error))
+		return FALSE;
+	if (!fu_uefi_device_cleanup_esp (device, error))
+		return FALSE;
+
+	return TRUE;
+}
+
+static gboolean
+fu_uefi_device_cleanup (FuDevice *device,
+			FwupdInstallFlags flags,
+			GError **error)
+{
+	FuUefiDevice *self = FU_UEFI_DEVICE (device);
+	if (self->automounted_esp) {
+		g_autofree gchar *guessed = NULL;
+		guessed = fu_uefi_guess_esp_path (error);
+		if (guessed == NULL)
+			return FALSE;
+		g_debug ("Unmounting ESP @ %s", guessed);
+		if (!fu_uefi_udisks_objpath_umount (guessed, error))
+			return FALSE;
+		self->automounted_esp = FALSE;
+		/* we will detect again if necessary */
+		fu_device_remove_metadata (device, "EspPath");
+	}
+
+	return TRUE;
+}
+
 static gboolean
 fu_uefi_device_write_firmware (FuDevice *device,
 			       FuFirmware *firmware,
@@ -475,7 +627,10 @@ fu_uefi_device_add_system_checksum (FuDevice *device, GError **error)
 	if (!fu_uefi_pcrs_setup (pcrs, &error_local)) {
 		if (g_error_matches (error_local,
 				     G_IO_ERROR,
-				     G_IO_ERROR_NOT_SUPPORTED)) {
+				     G_IO_ERROR_NOT_SUPPORTED) ||
+		    g_error_matches (error_local,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOT_FOUND)) {
 			g_debug ("%s", error_local->message);
 			return TRUE;
 		}
@@ -527,6 +682,7 @@ fu_uefi_device_probe (FuDevice *device, GError **error)
 	version_format = fu_device_get_version_format (device);
 	version = fu_common_version_from_uint32 (self->fw_version, version_format);
 	fu_device_set_version (device, version, version_format);
+	fu_device_set_version_raw (device, self->fw_version);
 	if (self->fw_version_lowest != 0) {
 		version_lowest = fu_common_version_from_uint32 (self->fw_version_lowest,
 							        version_format);
@@ -551,6 +707,7 @@ fu_uefi_device_probe (FuDevice *device, GError **error)
 	/* set the PCR0 as the device checksum */
 	if (self->kind == FU_UEFI_DEVICE_KIND_SYSTEM_FIRMWARE) {
 		g_autoptr(GError) error_local = NULL;
+		fu_device_add_flag (device, FWUPD_DEVICE_FLAG_CAN_VERIFY);
 		if (!fu_uefi_device_add_system_checksum (device, &error_local))
 			g_warning ("Failed to get PCR0s: %s", error_local->message);
 	}
@@ -566,6 +723,7 @@ fu_uefi_device_probe (FuDevice *device, GError **error)
 static void
 fu_uefi_device_init (FuUefiDevice *self)
 {
+	fu_device_set_protocol (FU_DEVICE (self), "org.uefi.capsule");
 }
 
 static void
@@ -586,7 +744,9 @@ fu_uefi_device_class_init (FuUefiDeviceClass *klass)
 	object_class->finalize = fu_uefi_device_finalize;
 	klass_device->to_string = fu_uefi_device_to_string;
 	klass_device->probe = fu_uefi_device_probe;
+	klass_device->prepare = fu_uefi_device_prepare;
 	klass_device->write_firmware = fu_uefi_device_write_firmware;
+	klass_device->cleanup = fu_uefi_device_cleanup;
 }
 
 FuUefiDevice *
