@@ -7,8 +7,9 @@
 #include "config.h"
 
 #include "fu-plugin-vfuncs.h"
+#include "fu-hash.h"
 
-#define MINIMUM_BATTERY_PERCENTAGE	10
+#define MINIMUM_BATTERY_PERCENTAGE	30
 
 struct FuPluginData {
 	GDBusProxy		*upower_proxy;
@@ -36,7 +37,6 @@ gboolean
 fu_plugin_startup (FuPlugin *plugin, GError **error)
 {
 	FuPluginData *data = fu_plugin_get_data (plugin);
-	g_autofree gchar *name_owner = NULL;
 	data->upower_proxy =
 		g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
 					       G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
@@ -50,15 +50,6 @@ fu_plugin_startup (FuPlugin *plugin, GError **error)
 		g_prefix_error (error, "failed to connect to upower: ");
 		return FALSE;
 	}
-	name_owner = g_dbus_proxy_get_name_owner (data->upower_proxy);
-	if (name_owner == NULL) {
-		g_set_error (error,
-			     FWUPD_ERROR,
-			     FWUPD_ERROR_NOT_SUPPORTED,
-			     "no owner for %s",
-			     g_dbus_proxy_get_name (data->upower_proxy));
-		return FALSE;
-	}
 	data->display_proxy =
 		g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
 					       G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
