@@ -476,6 +476,12 @@ fu_device_list_replace (FuDeviceList *self, FuDeviceItem *item, FuDevice *device
 		fu_device_add_flag (device, FWUPD_DEVICE_FLAG_ANOTHER_WRITE_REQUIRED);
 	}
 
+	/* device won't come back in right mode */
+	if (fu_device_has_flag (item->device, FWUPD_DEVICE_FLAG_WILL_DISAPPEAR)) {
+		g_debug ("copying will-disappear to new device");
+		fu_device_add_flag (device, FWUPD_DEVICE_FLAG_WILL_DISAPPEAR);
+	}
+
 	/* copy the parent if not already set */
 	if (fu_device_get_parent (item->device) != NULL &&
 	    fu_device_get_parent (device) == NULL) {
