@@ -17,8 +17,6 @@
 #include "fu-install-task.h"
 #include "fu-plugin.h"
 
-G_BEGIN_DECLS
-
 #define FU_TYPE_ENGINE (fu_engine_get_type ())
 G_DECLARE_FINAL_TYPE (FuEngine, fu_engine, FU, ENGINE, GObject)
 
@@ -32,6 +30,7 @@ G_DECLARE_FINAL_TYPE (FuEngine, fu_engine, FU, ENGINE, GObject)
 typedef enum {
 	FU_ENGINE_LOAD_FLAG_NONE		= 0,
 	FU_ENGINE_LOAD_FLAG_READONLY_FS		= 1 << 0,
+	FU_ENGINE_LOAD_FLAG_NO_ENUMERATE	= 1 << 1,
 	/*< private >*/
 	FU_ENGINE_LOAD_FLAG_LAST
 } FuEngineLoadFlags;
@@ -49,6 +48,7 @@ gboolean	 fu_engine_load_plugins			(FuEngine	*self,
 							 GError		**error);
 gboolean	 fu_engine_get_tainted			(FuEngine	*self);
 const gchar	*fu_engine_get_host_product		(FuEngine *self);
+const gchar	*fu_engine_get_host_machine_id		(FuEngine *self);
 FwupdStatus	 fu_engine_get_status			(FuEngine	*self);
 XbSilo		*fu_engine_get_silo_from_blob		(FuEngine	*self,
 							 GBytes		*blob_cab,
@@ -96,6 +96,10 @@ gboolean	 fu_engine_verify			(FuEngine	*self,
 gboolean	 fu_engine_verify_update		(FuEngine	*self,
 							 const gchar	*device_id,
 							 GError		**error);
+GBytes		*fu_engine_firmware_read		(FuEngine	*self,
+							 FuDevice	*device,
+							 FwupdInstallFlags flags,
+							 GError		**error);
 gboolean	 fu_engine_modify_remote		(FuEngine	*self,
 							 const gchar	*remote_id,
 							 const gchar	*key,
@@ -144,6 +148,9 @@ gboolean	 fu_engine_modify_config		(FuEngine	*self,
 							 const gchar	*key,
 							 const gchar	*value,
 							 GError		**error);
+GPtrArray	*fu_engine_get_firmware_gtype_ids	(FuEngine	*engine);
+GType		 fu_engine_get_firmware_gtype_by_id	(FuEngine	*engine,
+							 const gchar	*id);
 
 /* for the self tests */
 void		 fu_engine_add_device			(FuEngine	*self,
@@ -161,5 +168,9 @@ void		 fu_engine_set_silo			(FuEngine	*self,
 							 XbSilo		*silo);
 XbNode		*fu_engine_get_component_by_guids	(FuEngine	*self,
 							 FuDevice	*device);
-
-G_END_DECLS
+gboolean	 fu_engine_schedule_update		(FuEngine	*self,
+							 FuDevice	*device,
+							 FwupdRelease	*release,
+							 GBytes		*blob_cab,
+							 FwupdInstallFlags flags,
+							 GError		**error);
