@@ -11,7 +11,10 @@
 #include <fwupd.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
+#ifdef HAVE_GIO_UNIX
 #include <glib-unix.h>
+#endif
+#include <fcntl.h>
 #include <locale.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -22,7 +25,7 @@
 #include "fu-history.h"
 #include "fu-plugin-private.h"
 #include "fu-progressbar.h"
-#include "fu-smbios.h"
+#include "fu-smbios-private.h"
 #include "fu-util-common.h"
 #include "fu-debug.h"
 #include "fwupd-common-private.h"
@@ -39,6 +42,7 @@ typedef enum {
 	FU_UTIL_OPERATION_UNKNOWN,
 	FU_UTIL_OPERATION_UPDATE,
 	FU_UTIL_OPERATION_INSTALL,
+	FU_UTIL_OPERATION_READ,
 	FU_UTIL_OPERATION_LAST
 } FuUtilOperation;
 
@@ -49,6 +53,7 @@ struct FuUtilPrivate {
 	FuEngine		*engine;
 	FuProgressbar		*progressbar;
 	gboolean		 no_reboot_check;
+	gboolean		 no_safety_check;
 	gboolean		 prepare_blob;
 	gboolean		 cleanup_blob;
 	gboolean		 enable_json_state;
@@ -118,7 +123,7 @@ fu_util_start_engine (FuUtilPrivate *priv, FuEngineLoadFlags flags, GError **err
 
 #ifdef HAVE_SYSTEMD
 	if (!fu_systemd_unit_stop (fu_util_get_systemd_unit (), &error_local))
-		g_debug ("Failed top stop daemon: %s", error_local->message);
+		g_debug ("Failed to stop daemon: %s", error_local->message);
 #endif
 	if (!fu_engine_load (priv->engine, flags, error))
 		return FALSE;
@@ -169,6 +174,7 @@ fu_util_smbios_dump (FuUtilPrivate *priv, gchar **values, GError **error)
 	return TRUE;
 }
 
+#ifdef HAVE_GIO_UNIX
 static gboolean
 fu_util_sigint_cb (gpointer user_data)
 {
@@ -177,6 +183,7 @@ fu_util_sigint_cb (gpointer user_data)
 	g_cancellable_cancel (priv->cancellable);
 	return FALSE;
 }
+#endif
 
 static void
 fu_util_private_free (FuUtilPrivate *priv)
@@ -318,6 +325,7 @@ fu_util_get_updates (FuUtilPrivate *priv, gchar **values, GError **error)
 		return FALSE;
 	for (guint i = 0; i < devices->len; i++) {
 		FwupdDevice *dev = g_ptr_array_index (devices, i);
+		g_autofree gchar *upgrade_str = NULL;
 		g_autoptr(GPtrArray) rels = NULL;
 		g_autoptr(GError) error_local = NULL;
 		GNode *child;
@@ -333,7 +341,11 @@ fu_util_get_updates (FuUtilPrivate *priv, gchar **values, GError **error)
 					       fwupd_device_get_id (dev),
 					       &error_local);
 		if (rels == NULL) {
-			g_printerr ("%s\n", error_local->message);
+			/* TRANSLATORS: message letting the user know no device upgrade available
+			* %1 is the device name */
+			upgrade_str = g_strdup_printf (_("No upgrades for %s"),
+						      fwupd_device_get_name (dev));
+			g_printerr ("%s: %s\n", upgrade_str, error_local->message);
 			continue;
 		}
 		child = g_node_append_data (root, dev);
@@ -486,8 +498,6 @@ fu_util_prompt_for_device (FuUtilPrivate *priv, GError **error)
 		dev = g_ptr_array_index (devices, i);
 		if (!fu_util_filter_device (priv, FWUPD_DEVICE (dev)))
 			continue;
-		if (!fwupd_device_has_flag (FWUPD_DEVICE (dev), FWUPD_DEVICE_FLAG_SUPPORTED))
-			continue;
 		g_ptr_array_add (devices_filtered, g_object_ref (dev));
 	}
 
@@ -503,6 +513,8 @@ fu_util_prompt_for_device (FuUtilPrivate *priv, GError **error)
 	/* exactly one */
 	if (devices_filtered->len == 1) {
 		dev = g_ptr_array_index (devices_filtered, 0);
+		/* TRANSLATORS: Device has been chosen by the daemon for the user */
+		g_print ("%s: %s\n", _("Selected device"), fu_device_get_name (dev));
 		return g_object_ref (dev);
 	}
 
@@ -558,6 +570,11 @@ fu_util_update_device_changed_cb (FwupdClient *client,
 		str = g_strdup_printf (_("Installing on %s…"),
 				       fwupd_device_get_name (device));
 		fu_progressbar_set_title (priv->progressbar, str);
+	} else if (priv->current_operation == FU_UTIL_OPERATION_READ) {
+		/* TRANSLATORS: %1 is a device name  */
+		str = g_strdup_printf (_("Reading from %s…"),
+				       fwupd_device_get_name (device));
+		fu_progressbar_set_title (priv->progressbar, str);
 	} else {
 		g_warning ("no FuUtilOperation set");
 	}
@@ -660,6 +677,62 @@ fu_util_install_blob (FuUtilPrivate *priv, gchar **values, GError **error)
 	return fu_util_prompt_complete (priv->completion_flags, TRUE, error);
 }
 
+static gboolean
+fu_util_firmware_read (FuUtilPrivate *priv, gchar **values, GError **error)
+{
+	g_autoptr(FuDevice) device = NULL;
+	g_autoptr(GBytes) blob_empty = g_bytes_new (NULL, 0);
+	g_autoptr(GBytes) blob_fw = NULL;
+
+	/* invalid args */
+	if (g_strv_length (values) == 0) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_INVALID_ARGS,
+				     "Invalid arguments");
+		return FALSE;
+	}
+
+	/* file already exists */
+	if ((priv->flags & FWUPD_INSTALL_FLAG_FORCE) == 0 &&
+	    g_file_test (values[0], G_FILE_TEST_EXISTS)) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_INVALID_ARGS,
+				     "Filename already exists");
+		return FALSE;
+	}
+
+	/* write a zero length file to ensure the destination is writable to
+	 * avoid failing at the end of a potentially lengthy operation */
+	if (!fu_common_set_contents_bytes (values[0], blob_empty, error))
+		return FALSE;
+
+	/* load engine */
+	if (!fu_util_start_engine (priv, FU_ENGINE_LOAD_FLAG_NONE, error))
+		return FALSE;
+
+	/* get device */
+	if (g_strv_length (values) >= 2) {
+		device = fu_engine_get_device (priv->engine, values[1], error);
+		if (device == NULL)
+			return FALSE;
+	} else {
+		device = fu_util_prompt_for_device (priv, error);
+		if (device == NULL)
+			return FALSE;
+	}
+	priv->current_operation = FU_UTIL_OPERATION_READ;
+	g_signal_connect (priv->engine, "device-changed",
+			  G_CALLBACK (fu_util_update_device_changed_cb), priv);
+
+	/* dump firmware */
+	blob_fw = fu_engine_firmware_read (priv->engine, device, priv->flags, error);
+	if (blob_fw == NULL)
+		return FALSE;
+	return fu_common_set_contents_bytes (values[0], blob_fw, error);
+}
+
 static gint
 fu_util_install_task_sort_cb (gconstpointer a, gconstpointer b)
 {
@@ -671,7 +744,7 @@ fu_util_install_task_sort_cb (gconstpointer a, gconstpointer b)
 static gboolean
 fu_util_download_out_of_process (const gchar *uri, const gchar *fn, GError **error)
 {
-	const gchar *argv[][5] = { { "wget", uri, "-o", fn, NULL },
+	const gchar *argv[][5] = { { "wget", uri, "-O", fn, NULL },
 				   { "curl", uri, "--output", fn, NULL },
 				   { NULL } };
 	for (guint i = 0; argv[i][0] != NULL; i++) {
@@ -697,6 +770,8 @@ fu_util_download_if_required (FuUtilPrivate *priv, const gchar *perhapsfn, GErro
 
 	/* a local file */
 	uri = soup_uri_new (perhapsfn);
+	if (g_file_test (perhapsfn, G_FILE_TEST_EXISTS))
+		return g_strdup (perhapsfn);
 	if (uri == NULL)
 		return g_strdup (perhapsfn);
 
@@ -907,6 +982,13 @@ fu_util_update_all (FuUtilPrivate *priv, GError **error)
 			continue;
 		}
 
+		if (!priv->no_safety_check) {
+			if (!fu_util_prompt_warning (dev,
+						     fu_util_get_tree_title (priv),
+						     error))
+				return FALSE;
+		}
+
 		rel = g_ptr_array_index (rels, 0);
 		if (!fu_util_install_release (priv, rel, &error_local)) {
 			g_printerr ("%s\n", error_local->message);
@@ -944,6 +1026,22 @@ fu_util_update_by_id (FuUtilPrivate *priv, const gchar *device_id, GError **erro
 static gboolean
 fu_util_update (FuUtilPrivate *priv, gchar **values, GError **error)
 {
+	if (priv->flags & FWUPD_INSTALL_FLAG_ALLOW_OLDER) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_INVALID_ARGS,
+				     "--allow-older is not supported for this command");
+		return FALSE;
+	}
+
+	if (priv->flags & FWUPD_INSTALL_FLAG_ALLOW_REINSTALL) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_INVALID_ARGS,
+				     "--allow-reinstall is not supported for this command");
+		return FALSE;
+	}
+
 	if (g_strv_length (values) > 1) {
 		g_set_error_literal (error,
 				     FWUPD_ERROR,
@@ -990,15 +1088,6 @@ fu_util_detach (FuUtilPrivate *priv, gchar **values, GError **error)
 	if (!fu_util_start_engine (priv, FU_ENGINE_LOAD_FLAG_NONE, error))
 		return FALSE;
 
-	/* invalid args */
-	if (g_strv_length (values) == 0) {
-		g_set_error_literal (error,
-				     FWUPD_ERROR,
-				     FWUPD_ERROR_INVALID_ARGS,
-				     "Invalid arguments");
-		return FALSE;
-	}
-
 	/* get device */
 	if (g_strv_length (values) >= 1) {
 		device = fu_engine_get_device (priv->engine, values[0], error);
@@ -1027,15 +1116,6 @@ fu_util_attach (FuUtilPrivate *priv, gchar **values, GError **error)
 	if (!fu_util_start_engine (priv, FU_ENGINE_LOAD_FLAG_NONE, error))
 		return FALSE;
 
-	/* invalid args */
-	if (g_strv_length (values) == 0) {
-		g_set_error_literal (error,
-				     FWUPD_ERROR,
-				     FWUPD_ERROR_INVALID_ARGS,
-				     "Invalid arguments");
-		return FALSE;
-	}
-
 	/* get device */
 	if (g_strv_length (values) >= 1) {
 		device = fu_engine_get_device (priv->engine, values[0], error);
@@ -1090,13 +1170,10 @@ fu_util_activate (FuUtilPrivate *priv, gchar **values, GError **error)
 			has_pending = TRUE;
 		}
 	}
-	if (!has_pending) {
-		g_set_error_literal (error,
-				     FWUPD_ERROR,
-				     FWUPD_ERROR_NOTHING_TO_DO,
-				     "No firmware to activate");
-		return FALSE;
 
+	if (!has_pending) {
+    		g_printerr ("No firmware to activate\n");
+    		return TRUE;
 	}
 
 	/* load engine */
@@ -1314,6 +1391,107 @@ fu_util_monitor (FuUtilPrivate *priv, gchar **values, GError **error)
 	return TRUE;
 }
 
+static gboolean
+fu_util_get_firmware_types (FuUtilPrivate *priv, gchar **values, GError **error)
+{
+	g_autoptr(GPtrArray) firmware_types = NULL;
+
+	/* load engine */
+	if (!fu_engine_load (priv->engine, FU_ENGINE_LOAD_FLAG_NO_ENUMERATE, error))
+		return FALSE;
+
+	firmware_types = fu_engine_get_firmware_gtype_ids (priv->engine);
+	for (guint i = 0; i < firmware_types->len; i++) {
+		const gchar *id = g_ptr_array_index (firmware_types, i);
+		g_print ("%s\n", id);
+	}
+	if (firmware_types->len == 0) {
+		/* TRANSLATORS: nothing found */
+		g_print ("%s\n", _("No firmware IDs found"));
+		return TRUE;
+	}
+
+	return TRUE;
+}
+
+static gchar *
+fu_util_prompt_for_firmware_type (FuUtilPrivate *priv, GError **error)
+{
+	g_autoptr(GPtrArray) firmware_types = NULL;
+	guint idx;
+	firmware_types = fu_engine_get_firmware_gtype_ids (priv->engine);
+
+	/* TRANSLATORS: get interactive prompt */
+	g_print ("%s\n", _("Choose a firmware type:"));
+	/* TRANSLATORS: this is to abort the interactive prompt */
+	g_print ("0.\t%s\n", _("Cancel"));
+	for (guint i = 0; i < firmware_types->len; i++) {
+		const gchar *id = g_ptr_array_index (firmware_types, i);
+		g_print ("%u.\t%s\n", i + 1, id);
+	}
+	idx = fu_util_prompt_for_number (firmware_types->len);
+	if (idx == 0) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOTHING_TO_DO,
+				     "Request canceled");
+		return NULL;
+	}
+
+	return g_strdup (g_ptr_array_index (firmware_types, idx - 1));
+}
+
+static gboolean
+fu_util_firmware_parse (FuUtilPrivate *priv, gchar **values, GError **error)
+{
+	GType gtype;
+	g_autoptr(GBytes) blob = NULL;
+	g_autoptr(FuFirmware) firmware = NULL;
+	g_autofree gchar *firmware_type = NULL;
+	g_autofree gchar *str = NULL;
+
+	/* check args */
+	if (g_strv_length (values) == 0 || g_strv_length (values) > 2) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_INVALID_ARGS,
+				     "Invalid arguments: filename required");
+		return FALSE;
+	}
+
+	if (g_strv_length (values) == 2)
+		firmware_type = g_strdup (values[1]);
+
+	/* load file */
+	blob = fu_common_get_contents_bytes (values[0], error);
+	if (blob == NULL)
+		return FALSE;
+
+	/* load engine */
+	if (!fu_engine_load (priv->engine, FU_ENGINE_LOAD_FLAG_NO_ENUMERATE, error))
+		return FALSE;
+
+	/* find the GType to use */
+	if (firmware_type == NULL)
+		firmware_type = fu_util_prompt_for_firmware_type (priv, error);
+	if (firmware_type == NULL)
+		return FALSE;
+	gtype = fu_engine_get_firmware_gtype_by_id (priv->engine, firmware_type);
+	if (gtype == G_TYPE_INVALID) {
+		g_set_error (error,
+			     G_IO_ERROR,
+			     G_IO_ERROR_NOT_FOUND,
+			     "GType %s not supported", firmware_type);
+		return FALSE;
+	}
+	firmware = g_object_new (gtype, NULL);
+	if (!fu_firmware_parse (firmware, blob, priv->flags, error))
+		return FALSE;
+	str = fu_firmware_to_string (firmware);
+	g_print ("%s", str);
+	return TRUE;
+}
+
 static gboolean
 fu_util_verify_update (FuUtilPrivate *priv, gchar **values, GError **error)
 {
@@ -1363,10 +1541,52 @@ fu_util_get_history (FuUtilPrivate *priv, gchar **values, GError **error)
 
 	/* show each device */
 	for (guint i = 0; i < devices->len; i++) {
+		g_autoptr(GPtrArray) rels = NULL;
 		FwupdDevice *dev = g_ptr_array_index (devices, i);
+		FwupdRelease *rel;
+		const gchar *remote;
+		GNode *child;
+
 		if (!fu_util_filter_device (priv, dev))
 			continue;
-		g_node_append_data (root, dev);
+		child = g_node_append_data (root, dev);
+
+		rel = fwupd_device_get_release_default (dev);
+		if (rel == NULL)
+			continue;
+		remote = fwupd_release_get_remote_id (rel);
+
+		/* doesn't actually map to remote */
+		if (remote == NULL) {
+			g_node_append_data (child, rel);
+			continue;
+		}
+
+		/* try to lookup releases from client */
+		rels = fu_engine_get_releases (priv->engine, fwupd_device_get_id (dev), error);
+		if (rels == NULL)
+			return FALSE;
+
+		/* map to a release in client */
+		for (guint j = 0; j < rels->len; j++) {
+			FwupdRelease *rel2 = g_ptr_array_index (rels, j);
+			if (g_strcmp0 (remote,
+				       fwupd_release_get_remote_id (rel2)) != 0)
+				continue;
+			if (g_strcmp0 (fwupd_release_get_version (rel),
+				       fwupd_release_get_version (rel2)) != 0)
+				continue;
+			g_node_append_data (child, g_object_ref (rel2));
+			rel = NULL;
+			break;
+		}
+
+		/* didn't match anything */
+		if (rels->len == 0 || rel != NULL) {
+			g_node_append_data (child, rel);
+			continue;
+		}
+
 	}
 	fu_util_print_tree (root, title);
 
@@ -1394,7 +1614,7 @@ main (int argc, char *argv[])
 			_("Show client and daemon versions"), NULL },
 		{ "allow-reinstall", '\0', 0, G_OPTION_ARG_NONE, &allow_reinstall,
 			/* TRANSLATORS: command line option */
-			_("Allow re-installing existing firmware versions"), NULL },
+			_("Allow reinstalling existing firmware versions"), NULL },
 		{ "allow-older", '\0', 0, G_OPTION_ARG_NONE, &allow_older,
 			/* TRANSLATORS: command line option */
 			_("Allow downgrading firmware versions"), NULL },
@@ -1404,6 +1624,9 @@ main (int argc, char *argv[])
 		{ "no-reboot-check", '\0', 0, G_OPTION_ARG_NONE, &priv->no_reboot_check,
 			/* TRANSLATORS: command line option */
 			_("Do not check for reboot after update"), NULL },
+		{ "no-safety-check", '\0', 0, G_OPTION_ARG_NONE, &priv->no_safety_check,
+			/* TRANSLATORS: command line option */
+			_("Do not perform device safety checks"), NULL },
 		{ "show-all-devices", '\0', 0, G_OPTION_ARG_NONE, &priv->show_all_devices,
 			/* TRANSLATORS: command line option */
 			_("Show devices that are not updatable"), NULL },
@@ -1431,14 +1654,16 @@ main (int argc, char *argv[])
 
 	setlocale (LC_ALL, "");
 
-	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	bindtextdomain (GETTEXT_PACKAGE, FWUPD_LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+#ifdef HAVE_GETUID
 	/* ensure root user */
 	if (interactive && (getuid () != 0 || geteuid () != 0))
 		/* TRANSLATORS: we're poking around as a power user */
 		g_printerr ("%s\n", _("This program may only work correctly as root"));
+#endif
 
 	/* create helper object */
 	priv->loop = g_main_loop_new (NULL, FALSE);
@@ -1560,12 +1785,32 @@ main (int argc, char *argv[])
 		     /* TRANSLATORS: command description */
 		     _("Update the stored metadata with current contents"),
 		     fu_util_verify_update);
+	fu_util_cmd_array_add (cmd_array,
+		     "firmware-read",
+		     "FILENAME [DEVICE-ID]",
+		     /* TRANSLATORS: command description */
+		     _("Read a firmware blob from a device"),
+		     fu_util_firmware_read);
+	fu_util_cmd_array_add (cmd_array,
+		     "firmware-parse",
+		     "FILENAME [FIRMWARE_TYPE]",
+		     /* TRANSLATORS: command description */
+		     _("Parse and show details about a firmware file"),
+		     fu_util_firmware_parse);
+	fu_util_cmd_array_add (cmd_array,
+		     "get-firmware-types",
+		     NULL,
+		     /* TRANSLATORS: command description */
+		     _("List the available firmware types"),
+		     fu_util_get_firmware_types);
 
 	/* do stuff on ctrl+c */
 	priv->cancellable = g_cancellable_new ();
+#ifdef HAVE_GIO_UNIX
 	g_unix_signal_add_full (G_PRIORITY_DEFAULT,
 				SIGINT, fu_util_sigint_cb,
 				priv, NULL);
+#endif
 	g_signal_connect (priv->cancellable, "cancelled",
 			  G_CALLBACK (fu_util_cancelled_cb), priv);
 
@@ -1575,6 +1820,7 @@ main (int argc, char *argv[])
 	/* non-TTY consoles cannot answer questions */
 	if (!interactive) {
 		priv->no_reboot_check = TRUE;
+		priv->no_safety_check = TRUE;
 		fu_progressbar_set_interactive (priv->progressbar, FALSE);
 	}
 
