@@ -80,7 +80,7 @@ fu_util_print_data (const gchar *title, const gchar *msg)
 	g_print ("%s:", title);
 
 	/* pad */
-	title_len = strlen (title) + 1;
+	title_len = fu_common_strwidth (title) + 1;
 	lines = g_strsplit (msg, "\n", -1);
 	for (guint j = 0; lines[j] != NULL; j++) {
 		for (gsize i = title_len; i < 25; i++)
@@ -252,19 +252,31 @@ fu_util_get_user_cache_path (const gchar *fn)
 }
 
 gchar *
-fu_util_get_versions (void)
+fu_util_get_client_version (void)
 {
 	GString *string = g_string_new ("");
 
 	g_string_append_printf (string,
-				"client version:\t%i.%i.%i\n",
+				"%i.%i.%i",
 				FWUPD_MAJOR_VERSION,
 				FWUPD_MINOR_VERSION,
 				FWUPD_MICRO_VERSION);
-#ifdef FWUPD_GIT_DESCRIBE
+#ifdef FWUPD_DIRTY_VERSION
 	g_string_append_printf (string,
-				"checkout info:\t%s\n", FWUPD_GIT_DESCRIBE);
+				"-%i-%s",
+				FWUPD_DIRTY_VERSION,
+				FWUPD_COMMIT_VERSION);
 #endif
+	return g_string_free (string, FALSE);
+}
+
+gchar *
+fu_util_get_versions (void)
+{
+	GString *string = g_string_new ("");
+	g_autofree gchar *client_version = fu_util_get_client_version ();
+
+	g_string_append_printf (string, "client version:\t%s\n", client_version);
 	g_string_append_printf (string,
 				"compile-time dependency versions\n");
 	g_string_append_printf (string,
@@ -370,29 +382,75 @@ fu_util_update_reboot (GError **error)
 	return val != NULL;
 }
 
+gboolean
+fu_util_prompt_warning (FwupdDevice *device, const gchar *machine, GError **error)
+{
+	FwupdDeviceFlags flags;
+	g_autofree gchar *str = NULL;
+
+	/* device is already in bootloader mode */
+	flags = fwupd_device_get_flags (device);
+	if (flags & FWUPD_DEVICE_FLAG_IS_BOOTLOADER)
+		return TRUE;
+
+	/* device may reboot */
+	if ((flags & FWUPD_DEVICE_FLAG_USABLE_DURING_UPDATE) == 0) {
+		/* TRANSLATORS: warn the user before updating, %1 is a device name */
+		str = g_strdup_printf (_("%s and all connected devices may not be usable while updating."),
+					fwupd_device_get_name (device));
+	/* device can get bricked */
+	} else if ((flags & FWUPD_DEVICE_FLAG_SELF_RECOVERY) == 0) {
+		/* external device */
+		if ((flags & FWUPD_DEVICE_FLAG_INTERNAL) == 0) {
+			/* TRANSLATORS: warn the user before updating, %1 is a device name */
+			str = g_strdup_printf (_("%s must remain connected for the duration of the update to avoid damage."),
+						fwupd_device_get_name (device));
+		} else if (flags & FWUPD_DEVICE_FLAG_REQUIRE_AC) {
+			/* TRANSLATORS: warn the user before updating, %1 is a machine name */
+			str = g_strdup_printf (_("%s must remain plugged into a power source for the duration of the update to avoid damage."),
+						machine);
+		}
+	}
+	if (str != NULL) {
+		g_print ("%s %s [Y|n]: ",
+			str,
+			/* TRANSLATORS: prompt to apply the update */
+			_("Continue with update?"));
+		if (!fu_util_prompt_for_boolean (TRUE)) {
+			g_set_error_literal (error,
+					     FWUPD_ERROR,
+					     FWUPD_ERROR_NOTHING_TO_DO,
+					     "Request canceled");
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
 gboolean
 fu_util_prompt_complete (FwupdDeviceFlags flags, gboolean prompt, GError **error)
 {
 	if (flags & FWUPD_DEVICE_FLAG_NEEDS_SHUTDOWN) {
 		if (prompt) {
-			g_print ("\n%s %s [Y|n]: ",
+			g_print ("\n%s %s [y|N]: ",
 				 /* TRANSLATORS: explain why we want to shutdown */
 				 _("An update requires the system to shutdown to complete."),
 				 /* TRANSLATORS: shutdown to apply the update */
 				 _("Shutdown now?"));
-			if (!fu_util_prompt_for_boolean (TRUE))
+			if (!fu_util_prompt_for_boolean (FALSE))
 				return TRUE;
 		}
 		return fu_util_update_shutdown (error);
 	}
 	if (flags & FWUPD_DEVICE_FLAG_NEEDS_REBOOT) {
 		if (prompt) {
-			g_print ("\n%s %s [Y|n]: ",
+			g_print ("\n%s %s [y|N]: ",
 				 /* TRANSLATORS: explain why we want to reboot */
 				 _("An update requires a reboot to complete."),
 				 /* TRANSLATORS: reboot to apply the update */
 				 _("Restart now?"));
-			if (!fu_util_prompt_for_boolean (TRUE))
+			if (!fu_util_prompt_for_boolean (FALSE))
 				return TRUE;
 		}
 		return fu_util_update_reboot (error);
@@ -495,11 +553,11 @@ fu_util_cmd_array_to_string (GPtrArray *array)
 		FuUtilCmd *item = g_ptr_array_index (array, i);
 		g_string_append (string, "  ");
 		g_string_append (string, item->name);
-		len = strlen (item->name) + 2;
+		len = fu_common_strwidth (item->name) + 2;
 		if (item->arguments != NULL) {
 			g_string_append (string, " ");
 			g_string_append (string, item->arguments);
-			len += strlen (item->arguments) + 1;
+			len += fu_common_strwidth (item->arguments) + 1;
 		}
 		if (len < max_len) {
 			for (gsize j = len; j < max_len + 1; j++)
@@ -647,7 +705,7 @@ fu_util_strsplit_words (const gchar *text, guint line_len)
 	for (guint i = 0; tokens[i] != NULL; i++) {
 
 		/* current line plus new token is okay */
-		if (curline->len + strlen (tokens[i]) < line_len) {
+		if (curline->len + fu_common_strwidth (tokens[i]) < line_len) {
 			g_string_append_printf (curline, "%s ", tokens[i]);
 			continue;
 		}
@@ -677,15 +735,15 @@ fu_util_warning_box_line (const gchar *start,
 {
 	guint offset = 0;
 	if (start != NULL) {
-		offset += g_utf8_strlen (start, -1);
+		offset += fu_common_strwidth (start);
 		g_print ("%s", start);
 	}
 	if (text != NULL) {
-		offset += g_utf8_strlen (text, -1);
+		offset += fu_common_strwidth (text);
 		g_print ("%s", text);
 	}
 	if (end != NULL)
-		offset += g_utf8_strlen (end, -1);
+		offset += fu_common_strwidth (end);
 	for (guint i = offset; i < width; i++)
 		g_print ("%s", padding);
 	if (end != NULL)
@@ -876,6 +934,130 @@ fu_util_time_to_str (guint64 tmp)
 				(guint) tmp);
 }
 
+static gchar *
+fu_util_device_flag_to_string (guint64 device_flag)
+{
+	if (device_flag == FWUPD_DEVICE_FLAG_NONE) {
+		return NULL;
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_INTERNAL) {
+		/* TRANSLATORS: Device cannot be removed easily*/
+		return _("Internal device");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_UPDATABLE) {
+		/* TRANSLATORS: Device is updatable in this or any other mode */
+		return _("Updatable");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_ONLY_OFFLINE) {
+		/* TRANSLATORS: Update can only be done from offline mode */
+		return _("Update requires a reboot");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_REQUIRE_AC) {
+		/* TRANSLATORS: Must be plugged in to an outlet */
+		return _("Requires AC power");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_LOCKED) {
+		/* TRANSLATORS: Is locked and can be unlocked */
+		return _("Device is locked");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_SUPPORTED) {
+		/* TRANSLATORS: Is found in current metadata */
+		return _("Supported on remote server");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_NEEDS_BOOTLOADER) {
+		/* TRANSLATORS: Requires a bootloader mode to be manually enabled by the user */
+		return _("Requires a bootloader");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_NEEDS_REBOOT) {
+		/* TRANSLATORS: Requires a reboot to apply firmware or to reload hardware */
+		return _("Needs a reboot after installation");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_NEEDS_SHUTDOWN) {
+		/* TRANSLATORS: Requires system shutdown to apply firmware */
+		return _("Needs shutdown after installation");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_REPORTED) {
+		/* TRANSLATORS: Has been reported to a metadata server */
+		return _("Reported to remote server");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_NOTIFIED) {
+		/* TRANSLATORS: User has been notified */
+		return _("User has been notified");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_USE_RUNTIME_VERSION) {
+		/* skip */
+		return NULL;
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_INSTALL_PARENT_FIRST) {
+		/* TRANSLATORS: Install composite firmware on the parent before the child */
+		return _("Install to parent device first");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_IS_BOOTLOADER) {
+		/* TRANSLATORS: Is currently in bootloader mode */
+		return _("Is in bootloader mode");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG) {
+		/* TRANSLATORS: The hardware is waiting to be replugged */
+		return _("Hardware is waiting to be replugged");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_IGNORE_VALIDATION) {
+		/* TRANSLATORS: Ignore validation safety checks when flashing this device */
+		return _("Ignore validation safety checks");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_ANOTHER_WRITE_REQUIRED) {
+		/* skip */
+		return NULL;
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_NO_AUTO_INSTANCE_IDS) {
+		/* skip */
+		return NULL;
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION) {
+		/* TRANSLATORS: Device update needs to be separately activated */
+		return _("Device update needs activation");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_ENSURE_SEMVER) {
+		/* skip */
+		return NULL;
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_HISTORICAL) {
+		/* skip */
+		return NULL;
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_ONLY_SUPPORTED) {
+		/* skip */
+		return NULL;
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_WILL_DISAPPEAR) {
+		/* TRANSLATORS: Device will not return after update completes */
+		return _("Device will not re-appear after update completes");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_CAN_VERIFY) {
+		/* TRANSLATORS: Device supports some form of checksum verification */
+		return _("Cryptographic hash verification is available");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_CAN_VERIFY_IMAGE) {
+		/* skip */
+		return NULL;
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_DUAL_IMAGE) {
+		/* TRANSLATORS: Device supports a safety mechanism for flashing */
+		return _("Device stages updates");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_SELF_RECOVERY) {
+		/* TRANSLATORS: Device supports a safety mechanism for flashing */
+		return _("Device can recover flash failures");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_USABLE_DURING_UPDATE) {
+		/* TRANSLATORS: Device remains usable during update */
+		return _("Device is usable for the duration of the update");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_UNKNOWN) {
+		return NULL;
+	}
+	return NULL;
+}
+
 gchar *
 fu_util_device_to_string (FwupdDevice *dev, guint idt)
 {
@@ -886,8 +1068,8 @@ fu_util_device_to_string (FwupdDevice *dev, guint idt)
 	const gchar *tmp;
 	const gchar *tmp2;
 	guint64 flags = fwupd_device_get_flags (dev);
+	guint64 modified = fwupd_device_get_modified (dev);
 	g_autoptr(GHashTable) ids = NULL;
-	g_autoptr(GString) flags_str = g_string_new (NULL);
 
 	/* some fields are intentionally not included and are only shown in --verbose */
 	if (g_getenv ("FWUPD_VERBOSE") != NULL) {
@@ -929,8 +1111,13 @@ fu_util_device_to_string (FwupdDevice *dev, guint idt)
 	/* versions */
 	tmp = fwupd_device_get_version (dev);
 	if (tmp != NULL) {
-		/* TRANSLATORS: version number of current firmware */
-		fu_common_string_append_kv (str, idt + 1, _("Current version"), tmp);
+		if (flags & FWUPD_DEVICE_FLAG_HISTORICAL) {
+			/* TRANSLATORS: version number of previous firmware */
+			fu_common_string_append_kv (str, idt + 1, _("Previous version"), tmp);
+		} else {
+			/* TRANSLATORS: version number of current firmware */
+			fu_common_string_append_kv (str, idt + 1, _("Current version"), tmp);
+		}
 	}
 	tmp = fwupd_device_get_version_lowest (dev);
 	if (tmp != NULL) {
@@ -990,16 +1177,14 @@ fu_util_device_to_string (FwupdDevice *dev, guint idt)
 		fu_common_string_append_kv (str, idt + 1, _("Update Message"), tmp);
 	}
 
-	for (guint i = 0; i < 64; i++) {
-		if ((flags & ((guint64) 1 << i)) == 0)
-			continue;
-		g_string_append_printf (flags_str, "%s|",
-					fwupd_device_flag_to_string ((guint64) 1 << i));
-	}
-	if (flags_str->len > 0) {
-		g_string_truncate (flags_str, flags_str->len - 1);
-		/* TRANSLATORS: device properties */
-		fu_common_string_append_kv (str, idt + 1, _("Flags"), flags_str->str);
+	/* modified date: for history devices */
+	if (modified > 0) {
+		g_autoptr(GDateTime) date = NULL;
+		g_autofree gchar *time_str = NULL;
+		date = g_date_time_new_from_unix_utc (modified);
+		time_str = g_date_time_format (date, "%F %R");
+		/* TRANSLATORS: the original time/date the device was modified */
+		fu_common_string_append_kv (str, idt +1, _("Last modified"), time_str);
 	}
 
 	/* all GUIDs for this hardware, with IDs if available */
@@ -1028,12 +1213,50 @@ fu_util_device_to_string (FwupdDevice *dev, guint idt)
 			fu_common_string_append_kv (str, idt + 1, "", guid_src);
 		}
 	}
+
+	/* TRANSLATORS: description of device ability */
+	tmp = _("Device Flags");
+	for (guint i = 0; i < 64; i++) {
+		if ((flags & ((guint64) 1 << i)) == 0)
+			continue;
+		tmp2 = fu_util_device_flag_to_string ((guint64) 1 << i);
+		if (tmp2 == NULL)
+			continue;
+		/* header */
+		if (tmp != NULL) {
+			g_autofree gchar *bullet = NULL;
+			bullet = g_strdup_printf ("• %s", tmp2);
+			fu_common_string_append_kv (str, idt + 1, tmp, bullet);
+			tmp = NULL;
+		} else {
+			g_autofree gchar *bullet = NULL;
+			bullet = g_strdup_printf ("• %s", tmp2);
+			fu_common_string_append_kv (str, idt + 1, "", bullet);
+		}
+	}
+
 	return g_string_free (str, FALSE);
 }
 
+static const gchar *
+fu_util_license_to_string (const gchar *license)
+{
+	if (license == NULL) {
+		/* TRANSLATORS: we don't know the license of the update */
+		return _("Unknown");
+	}
+	if (g_strcmp0 (license, "LicenseRef-proprietary") == 0 ||
+	    g_strcmp0 (license, "proprietary") == 0) {
+		/* TRANSLATORS: a non-free software license */
+		return _("Proprietary");
+	}
+	return license;
+}
+
 gchar *
 fu_util_release_to_string (FwupdRelease *rel, guint idt)
 {
+	GPtrArray *issues = fwupd_release_get_issues (rel);
 	GString *str = g_string_new (NULL);
 	guint64 flags = fwupd_release_get_flags (rel);
 	g_autoptr(GString) flags_str = g_string_new (NULL);
@@ -1043,7 +1266,7 @@ fu_util_release_to_string (FwupdRelease *rel, guint idt)
 	fu_common_string_append_kv (str, idt, fwupd_release_get_name (rel), NULL);
 
 	/* TRANSLATORS: version number of new firmware */
-	fu_common_string_append_kv (str, idt + 1 , _("Version"),
+	fu_common_string_append_kv (str, idt + 1 , _("New version"),
 				    fwupd_release_get_version (rel));
 
 	if (fwupd_release_get_remote_id (rel) != NULL) {
@@ -1056,11 +1279,14 @@ fu_util_release_to_string (FwupdRelease *rel, guint idt)
 		fu_common_string_append_kv (str, idt + 1, _("Summary"),
 					    fwupd_release_get_summary (rel));
 	}
-	if (fwupd_release_get_license (rel) != NULL) {
-		/* TRANSLATORS: e.g. GPLv2+, Non free etc */
-		fu_common_string_append_kv (str, idt + 1, _("License"),
-					    fwupd_release_get_license (rel));
+	if (fwupd_release_get_name_variant_suffix (rel) != NULL) {
+		/* TRANSLATORS: one line variant of release (e.g. 'Prerelease' or 'China') */
+		fu_common_string_append_kv (str, idt + 1, _("Variant"),
+					    fwupd_release_get_name_variant_suffix (rel));
 	}
+	/* TRANSLATORS: e.g. GPLv2+, Proprietary etc */
+	fu_common_string_append_kv (str, idt + 1, _("License"),
+				    fu_util_license_to_string (fwupd_release_get_license (rel)));
 	if (fwupd_release_get_size (rel) != 0) {
 		g_autofree gchar *tmp = NULL;
 		tmp = g_format_size (fwupd_release_get_size (rel));
@@ -1099,10 +1325,7 @@ fu_util_release_to_string (FwupdRelease *rel, guint idt)
 		g_string_append_printf (flags_str, "%s|",
 					fwupd_release_flag_to_string ((guint64) 1 << i));
 	}
-	if (flags_str->len == 0) {
-		/* TRANSLATORS: release properties */
-		fu_common_string_append_kv (str, idt + 1, _("Flags"), fwupd_release_flag_to_string (0));
-	} else {
+	if (flags_str->len > 0) {
 		g_string_truncate (flags_str, flags_str->len - 1);
 		/* TRANSLATORS: release properties */
 		fu_common_string_append_kv (str, idt + 1, _("Flags"), flags_str->str);
@@ -1113,6 +1336,15 @@ fu_util_release_to_string (FwupdRelease *rel, guint idt)
 		/* TRANSLATORS: multiline description of device */
 		fu_common_string_append_kv (str, idt + 1, _("Description"), desc);
 	}
+	for (guint i = 0; i < issues->len; i++) {
+		const gchar *issue = g_ptr_array_index (issues, i);
+		if (i == 0) {
+			/* TRANSLATORS: issue fixed with the release, e.g. CVE */
+			fu_common_string_append_kv (str, idt + 1, ngettext ("Issue", "Issues", issues->len), issue);
+		} else {
+			fu_common_string_append_kv (str, idt + 1, "", issue);
+		}
+	}
 
 	return g_string_free (str, FALSE);
 }
@@ -1196,7 +1428,7 @@ fu_util_remote_to_string (FwupdRemote *remote, guint idt)
 	}
 	tmp = fwupd_remote_get_password (remote);
 	if (tmp != NULL) {
-		g_autofree gchar *hidden = g_strnfill (strlen (tmp), '*');
+		g_autofree gchar *hidden = g_strnfill (fu_common_strwidth (tmp), '*');
 		/* TRANSLATORS: remote filename base */
 		fu_common_string_append_kv (str, idt + 1, _("Password"), hidden);
 	}
@@ -1229,6 +1461,9 @@ fu_util_remote_to_string (FwupdRemote *remote, guint idt)
 	if (tmp != NULL) {
 		/* TRANSLATORS: URI to send success/failure reports */
 		fu_common_string_append_kv (str, idt + 1, _("Report URI"), tmp);
+		/* TRANSLATORS: Boolean value to automatically send reports */
+		fu_common_string_append_kv (str, idt + 1, _("Automatic Reporting"),
+					    fwupd_remote_get_automatic_reports (remote) ? "true" : "false");
 	}
 
 	return g_string_free (str, FALSE);
