@@ -15,8 +15,9 @@
 #include <gio/gio.h>
 #include <gio/gunixfdlist.h>
 #include <glib/gi18n.h>
+#ifdef HAVE_GIO_UNIX
 #include <glib-unix.h>
-#include <gudev/gudev.h>
+#endif
 #include <json-glib/json-glib.h>
 #include <locale.h>
 #include <stdlib.h>
@@ -35,6 +36,14 @@
 /* custom return code */
 #define EXIT_NOTHING_TO_DO		2
 
+typedef enum {
+	FU_UTIL_HISTORY_DO_NOTHING,
+	FU_UTIL_HISTORY_NEVER,
+	FU_UTIL_HISTORY_PROMPT,
+	FU_UTIL_HISTORY_AUTOMATIC,
+	FU_UTIL_HISTORY_LAST,
+} FuUtilHistoryAction;
+
 typedef enum {
 	FU_UTIL_OPERATION_UNKNOWN,
 	FU_UTIL_OPERATION_UPDATE,
@@ -54,6 +63,7 @@ struct FuUtilPrivate {
 	gboolean		 no_metadata_check;
 	gboolean		 no_reboot_check;
 	gboolean		 no_unreported_check;
+	gboolean		 no_safety_check;
 	gboolean		 assume_yes;
 	gboolean		 sign;
 	gboolean		 show_all_devices;
@@ -135,13 +145,16 @@ fu_util_update_device_changed_cb (FwupdClient *client,
 static gboolean
 fu_util_filter_device (FuUtilPrivate *priv, FwupdDevice *dev)
 {
-	if (priv->filter_include != FWUPD_DEVICE_FLAG_NONE) {
-		if (!fwupd_device_has_flag (dev, priv->filter_include))
-			return FALSE;
-	}
-	if (priv->filter_exclude != FWUPD_DEVICE_FLAG_NONE) {
-		if (fwupd_device_has_flag (dev, priv->filter_exclude))
-			return FALSE;
+	for (guint i = 0; i < 64; i++) {
+		FwupdDeviceFlags flag = 1LLU << i;
+		if (priv->filter_include & flag) {
+			if (!fwupd_device_has_flag (dev, flag))
+				return FALSE;
+		}
+		if (priv->filter_exclude & flag) {
+			if (fwupd_device_has_flag (dev, flag))
+				return FALSE;
+		}
 	}
 	return TRUE;
 }
@@ -165,8 +178,6 @@ fu_util_prompt_for_device (FuUtilPrivate *priv, GError **error)
 		dev = g_ptr_array_index (devices, i);
 		if (!fu_util_filter_device (priv, dev))
 			continue;
-		if (!fwupd_device_has_flag (dev, FWUPD_DEVICE_FLAG_SUPPORTED))
-			continue;
 		g_ptr_array_add (devices_filtered, dev);
 	}
 
@@ -182,6 +193,8 @@ fu_util_prompt_for_device (FuUtilPrivate *priv, GError **error)
 	/* exactly one */
 	if (devices_filtered->len == 1) {
 		dev = g_ptr_array_index (devices_filtered, 0);
+		/* TRANSLATORS: Device has been chosen by the daemon for the user */
+		g_print ("%s: %s\n", _("Selected device"), fwupd_device_get_name (dev));
 		return g_object_ref (dev);
 	}
 
@@ -208,6 +221,85 @@ fu_util_prompt_for_device (FuUtilPrivate *priv, GError **error)
 	return g_object_ref (dev);
 }
 
+static gboolean
+fu_util_maybe_enable_automatic (FuUtilPrivate *priv, GPtrArray *remotes, GError **error)
+{
+	guint idx;
+
+	/* TRANSLATORS: Display a message asking every time an update is performed */
+	g_print ("%d.\t%s\n",
+		 FU_UTIL_HISTORY_DO_NOTHING,
+		 ngettext ("Do not upload report at this time, but prompt again for future updates",
+			   "Do not upload reports at this time, but prompt again for future updates",
+			   remotes->len));
+	/* TRANSLATORS: Display a message asking every time an update is performed */
+	g_print ("%d.\t%s\n",
+		 FU_UTIL_HISTORY_NEVER,
+		 ngettext ("Do not upload report, and never ask to upload reports for future updates",
+			   "Do not upload reports, and never ask to upload reports for future updates",
+			   remotes->len));
+	/* TRANSLATORS: Display a message asking every time an update is performed */
+	g_print ("%d.\t%s\n",
+		 FU_UTIL_HISTORY_PROMPT,
+		 ngettext ("Upload report just this one time, but prompt again for future updates",
+			   "Upload reports just this one time, but prompt again for future updates",
+			   remotes->len));
+	/* TRANSLATORS: Display a message asking every time an update is performed */
+	g_print ("%d.\t%s\n",
+		 FU_UTIL_HISTORY_AUTOMATIC,
+		 ngettext ("Upload report this time and automatically upload reports after completing future updates",
+			   "Upload reports this time and automatically upload reports after completing future updates",
+			   remotes->len));
+	idx = fu_util_prompt_for_number (FU_UTIL_HISTORY_LAST - 1);
+
+	switch (idx) {
+	case FU_UTIL_HISTORY_NEVER:
+		for (guint i = 0; i < remotes->len; i++) {
+			FwupdRemote *remote = g_ptr_array_index (remotes, i);
+			const gchar *remote_id = fwupd_remote_get_id (remote);
+			if (fwupd_remote_get_report_uri (remote) == NULL)
+				continue;
+			if (!fwupd_client_modify_remote (priv->client,
+							 remote_id, "ReportURI", "",
+							 NULL, error))
+				return FALSE;
+		}
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOTHING_TO_DO,
+				     "Reporting disabled");
+		return FALSE;
+	case FU_UTIL_HISTORY_DO_NOTHING:
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOTHING_TO_DO,
+				     "Request canceled");
+		return FALSE;
+	case FU_UTIL_HISTORY_AUTOMATIC:
+		for (guint i = 0; i < remotes->len; i++) {
+			FwupdRemote *remote = g_ptr_array_index (remotes, i);
+			const gchar *remote_id = fwupd_remote_get_id (remote);
+			if (fwupd_remote_get_report_uri (remote) == NULL)
+				continue;
+			if (fwupd_remote_get_automatic_reports (remote))
+				continue;
+			if (!fwupd_client_modify_remote (priv->client,
+							 remote_id, "AutomaticReports", "true",
+							 NULL, error))
+				return FALSE;
+		}
+	/* fallthrough */
+	default:
+		return TRUE;
+	}
+
+	g_set_error_literal (error,
+				FWUPD_ERROR,
+				FWUPD_ERROR_NOTHING_TO_DO,
+				"no option selected");
+	return FALSE;
+}
+
 static gboolean
 fu_util_perhaps_show_unreported (FuUtilPrivate *priv, GError **error)
 {
@@ -215,6 +307,9 @@ fu_util_perhaps_show_unreported (FuUtilPrivate *priv, GError **error)
 	g_autoptr(GPtrArray) devices = NULL;
 	g_autoptr(GPtrArray) devices_failed = g_ptr_array_new ();
 	g_autoptr(GPtrArray) devices_success = g_ptr_array_new ();
+	g_autoptr(GPtrArray) remotes = NULL;
+	g_autoptr(GHashTable) remote_id_uri_map = NULL;
+	gboolean all_automatic = FALSE;
 
 	/* we don't want to ask anything */
 	if (priv->no_unreported_check) {
@@ -231,23 +326,69 @@ fu_util_perhaps_show_unreported (FuUtilPrivate *priv, GError **error)
 		return FALSE;
 	}
 
+	/* create a map of RemoteID to RemoteURI */
+	remotes = fwupd_client_get_remotes (priv->client, NULL, error);
+	if (remotes == NULL)
+		return FALSE;
+	remote_id_uri_map = g_hash_table_new (g_str_hash, g_str_equal);
+	for (guint i = 0; i < remotes->len; i++) {
+		FwupdRemote *remote = g_ptr_array_index (remotes, i);
+		gboolean remote_automatic;
+		if (fwupd_remote_get_id (remote) == NULL)
+			continue;
+		if (fwupd_remote_get_report_uri (remote) == NULL)
+			continue;
+		g_debug ("adding %s for %s",
+			 fwupd_remote_get_report_uri (remote),
+			 fwupd_remote_get_id (remote));
+		g_hash_table_insert (remote_id_uri_map,
+				     (gpointer) fwupd_remote_get_id (remote),
+				     (gpointer) fwupd_remote_get_report_uri (remote));
+		remote_automatic = fwupd_remote_get_automatic_reports (remote);
+		g_debug ("%s is %d", fwupd_remote_get_title (remote), remote_automatic);
+		if (remote_automatic && !all_automatic)
+			all_automatic = TRUE;
+		if (!remote_automatic && all_automatic) {
+			all_automatic = FALSE;
+			break;
+		}
+	}
+
+	/* check that they can be reported */
 	for (guint i = 0; i < devices->len; i++) {
 		FwupdDevice *dev = g_ptr_array_index (devices, i);
+		FwupdRelease *rel = fwupd_device_get_release_default (dev);
+		const gchar *remote_id;
+		const gchar *remote_uri;
+
 		if (!fu_util_filter_device (priv, dev))
 			continue;
 		if (fwupd_device_has_flag (dev, FWUPD_DEVICE_FLAG_REPORTED))
 			continue;
 		if (!fwupd_device_has_flag (dev, FWUPD_DEVICE_FLAG_SUPPORTED))
 			continue;
-		switch (fwupd_device_get_update_state (dev)) {
-		case FWUPD_UPDATE_STATE_FAILED:
+
+		/* find the RemoteURI to use for the device */
+		remote_id = fwupd_release_get_remote_id (rel);
+		if (remote_id == NULL) {
+			g_debug ("%s has no RemoteID", fwupd_device_get_id (dev));
+			continue;
+		}
+		remote_uri = g_hash_table_lookup (remote_id_uri_map, remote_id);
+		if (remote_uri == NULL) {
+			g_debug ("%s has no RemoteURI", remote_id);
+			continue;
+		}
+
+		/* only send success and failure */
+		if (fwupd_device_get_update_state (dev) == FWUPD_UPDATE_STATE_FAILED) {
 			g_ptr_array_add (devices_failed, dev);
-			break;
-		case FWUPD_UPDATE_STATE_SUCCESS:
+		} else if (fwupd_device_get_update_state (dev) == FWUPD_UPDATE_STATE_SUCCESS) {
 			g_ptr_array_add (devices_success, dev);
-			break;
-		default:
-			break;
+		} else {
+			g_debug ("ignoring %s with UpdateState %s",
+				 fwupd_device_get_id (dev),
+				 fwupd_update_state_to_string (fwupd_device_get_update_state (dev)));
 		}
 	}
 
@@ -257,8 +398,9 @@ fu_util_perhaps_show_unreported (FuUtilPrivate *priv, GError **error)
 		return TRUE;
 	}
 
+	g_debug ("All automatic: %d", all_automatic);
 	/* show the success and failures */
-	if (!priv->assume_yes) {
+	if (!priv->assume_yes && !all_automatic) {
 
 		/* delimit */
 		g_print ("________________________________________________\n");
@@ -292,7 +434,7 @@ fu_util_perhaps_show_unreported (FuUtilPrivate *priv, GError **error)
 		}
 
 		/* ask for permission */
-		g_print ("\n%s\n%s (%s) [Y|n]: ",
+		g_print ("\n%s\n%s (%s):\n",
 			 /* TRANSLATORS: explain why we want to upload */
 			 _("Uploading firmware reports helps hardware vendors"
 			   " to quickly identify failing and successful updates"
@@ -301,8 +443,8 @@ fu_util_perhaps_show_unreported (FuUtilPrivate *priv, GError **error)
 			 _("Upload report now?"),
 			 /* TRANSLATORS: metadata is downloaded from the Internet */
 			 _("Requires internet connection"));
-		if (!fu_util_prompt_for_boolean (TRUE))
-			return TRUE;
+		if (!fu_util_maybe_enable_automatic (priv, remotes, error))
+			return FALSE;
 	}
 
 	/* success */
@@ -426,6 +568,8 @@ fu_util_download_if_required (FuUtilPrivate *priv, const gchar *perhapsfn, GErro
 	g_autoptr(SoupURI) uri = NULL;
 
 	/* a local file */
+	if (g_file_test (perhapsfn, G_FILE_TEST_EXISTS))
+		return g_strdup (perhapsfn);
 	uri = soup_uri_new (perhapsfn);
 	if (uri == NULL)
 		return g_strdup (perhapsfn);
@@ -442,9 +586,13 @@ fu_util_download_if_required (FuUtilPrivate *priv, const gchar *perhapsfn, GErro
 static void
 fu_util_display_current_message (FuUtilPrivate *priv)
 {
-	if (priv->current_message == NULL)
+	if (priv->current_message == NULL) {
+		/* TRANSLATORS: success message */
+		g_print ("%s\n", _("Successfully installed firmware"));
 		return;
-	g_print ("%s\n", priv->current_message);
+	}
+	/* TRANSLATORS: success message */
+	g_print ("%s: %s\n", _("Successfully installed firmware"), priv->current_message);
 	g_clear_pointer (&priv->current_message, g_free);
 }
 
@@ -523,8 +671,8 @@ fu_util_clear_history (FuUtilPrivate *priv, gchar **values, GError **error)
 }
 
 static gboolean
-fu_util_report_history_for_uri (FuUtilPrivate *priv,
-				const gchar *report_uri,
+fu_util_report_history_for_remote (FuUtilPrivate *priv,
+				const gchar *remote_id,
 				GPtrArray *devices,
 				GError **error)
 {
@@ -532,10 +680,12 @@ fu_util_report_history_for_uri (FuUtilPrivate *priv,
 	JsonObject *json_object;
 	const gchar *server_msg = NULL;
 	guint status_code;
+	const gchar *report_uri;
 	g_autofree gchar *data = NULL;
 	g_autofree gchar *sig = NULL;
 	g_autoptr(JsonParser) json_parser = NULL;
 	g_autoptr(SoupMessage) msg = NULL;
+	g_autoptr(FwupdRemote) remote = NULL;
 
 	/* convert to JSON */
 	data = fwupd_build_history_report_json (devices, error);
@@ -551,8 +701,14 @@ fu_util_report_history_for_uri (FuUtilPrivate *priv,
 			return FALSE;
 	}
 
+	remote = fwupd_client_get_remote_by_id (priv->client, remote_id,
+						NULL, error);
+	if (remote == NULL)
+		return FALSE;
+	report_uri = fwupd_remote_get_report_uri (remote);
+
 	/* ask for permission */
-	if (!priv->assume_yes) {
+	if (!priv->assume_yes && !fwupd_remote_get_automatic_reports (remote)) {
 		fu_util_print_data (_("Target"), report_uri);
 		fu_util_print_data (_("Payload"), data);
 		if (sig != NULL)
@@ -670,9 +826,10 @@ fu_util_report_history (FuUtilPrivate *priv, gchar **values, GError **error)
 {
 	g_autoptr(GHashTable) remote_id_uri_map = NULL;
 	g_autoptr(GHashTable) report_map = NULL;
-	g_autoptr(GList) uris = NULL;
+	g_autoptr(GList) ids = NULL;
 	g_autoptr(GPtrArray) devices = NULL;
 	g_autoptr(GPtrArray) remotes = NULL;
+	g_autoptr(GString) str = g_string_new (NULL);
 
 	/* set up networking */
 	if (priv->soup_session == NULL) {
@@ -723,7 +880,6 @@ fu_util_report_history (FuUtilPrivate *priv, gchar **values, GError **error)
 			if (!fwupd_device_has_flag (dev, FWUPD_DEVICE_FLAG_SUPPORTED))
 				continue;
 		}
-
 		/* only send success and failure */
 		if (fwupd_device_get_update_state (dev) != FWUPD_UPDATE_STATE_FAILED &&
 		    fwupd_device_get_update_state (dev) != FWUPD_UPDATE_STATE_SUCCESS) {
@@ -746,12 +902,12 @@ fu_util_report_history (FuUtilPrivate *priv, gchar **values, GError **error)
 		}
 
 		/* add this to the hash map */
-		devices_tmp = g_hash_table_lookup (report_map, remote_uri);
+		devices_tmp = g_hash_table_lookup (report_map, remote_id);
 		if (devices_tmp == NULL) {
 			devices_tmp = g_ptr_array_new ();
-			g_hash_table_insert (report_map, g_strdup (remote_uri), devices_tmp);
+			g_hash_table_insert (report_map, g_strdup (remote_id), devices_tmp);
 		}
-		g_debug ("using %s for %s", remote_uri, fwupd_device_get_id (dev));
+		g_debug ("using %s for %s", remote_id, fwupd_device_get_id (dev));
 		g_ptr_array_add (devices_tmp, dev);
 	}
 
@@ -765,11 +921,11 @@ fu_util_report_history (FuUtilPrivate *priv, gchar **values, GError **error)
 	}
 
 	/* process each uri */
-	uris = g_hash_table_get_keys (report_map);
-	for (GList *l = uris; l != NULL; l = l->next) {
-		const gchar *uri = l->data;
-		GPtrArray *devices_tmp = g_hash_table_lookup (report_map, uri);
-		if (!fu_util_report_history_for_uri (priv, uri, devices_tmp, error))
+	ids = g_hash_table_get_keys (report_map);
+	for (GList *l = ids; l != NULL; l = l->next) {
+		const gchar *id = l->data;
+		GPtrArray *devices_tmp = g_hash_table_lookup (report_map, id);
+		if (!fu_util_report_history_for_remote (priv, id, devices_tmp, error))
 			return FALSE;
 	}
 
@@ -788,6 +944,13 @@ fu_util_report_history (FuUtilPrivate *priv, gchar **values, GError **error)
 			return FALSE;
 	}
 
+	/* TRANSLATORS: success message -- where the user has uploaded
+	 * success and/or failure reports to the remote server */
+	g_string_append_printf (str, ngettext ("Successfully uploaded %u report",
+					       "Successfully uploaded %u reports",
+					       g_hash_table_size (report_map)),
+					       g_hash_table_size (report_map));
+	g_print ("%s\n", str->str);
 	return TRUE;
 }
 
@@ -805,11 +968,61 @@ fu_util_get_history (FuUtilPrivate *priv, gchar **values, GError **error)
 
 	/* show each device */
 	for (guint i = 0; i < devices->len; i++) {
+		g_autoptr(GPtrArray) rels = NULL;
 		FwupdDevice *dev = g_ptr_array_index (devices, i);
+		FwupdRelease *rel;
+		const gchar *remote;
+		GNode *child;
+		g_autoptr(GError) error_local = NULL;
+
 		if (!fu_util_filter_device (priv, dev))
 			continue;
-		g_node_append_data (root, dev);
+		child = g_node_append_data (root, dev);
+
+		rel = fwupd_device_get_release_default (dev);
+		if (rel == NULL)
+			continue;
+		remote = fwupd_release_get_remote_id (rel);
+
+		/* doesn't actually map to remote */
+		if (remote == NULL) {
+			g_node_append_data (child, rel);
+			continue;
+		}
+
+		/* try to lookup releases from client */
+		rels = fwupd_client_get_releases (priv->client,
+						  fwupd_device_get_id (dev),
+						  NULL, &error_local);
+		if (rels == NULL) {
+			g_debug ("failed to get releases for %s: %s",
+				 fwupd_device_get_id (dev),
+				 error_local->message);
+			g_node_append_data (child, rel);
+			continue;
+		}
+
+		/* map to a release in client */
+		for (guint j = 0; j < rels->len; j++) {
+			FwupdRelease *rel2 = g_ptr_array_index (rels, j);
+			if (g_strcmp0 (remote,
+				       fwupd_release_get_remote_id (rel2)) != 0)
+				continue;
+			if (g_strcmp0 (fwupd_release_get_version (rel),
+				       fwupd_release_get_version (rel2)) != 0)
+				continue;
+			g_node_append_data (child, g_object_ref (rel2));
+			rel = NULL;
+			break;
+		}
+
+		/* didn't match anything */
+		if (rels->len == 0 || rel != NULL) {
+			g_node_append_data (child, rel);
+			continue;
+		}
 	}
+
 	fu_util_print_tree (root, title);
 
 	return TRUE;
@@ -855,51 +1068,24 @@ fu_util_clear_offline (FuUtilPrivate *priv, gchar **values, GError **error)
 	return fu_history_remove_all_with_state (history, FWUPD_UPDATE_STATE_PENDING, error);
 }
 
-static gboolean
-fu_util_verify_update_all (FuUtilPrivate *priv, GError **error)
-{
-	g_autoptr(GPtrArray) devs = NULL;
-
-	/* get devices from daemon */
-	devs = fwupd_client_get_devices (priv->client, NULL, error);
-	if (devs == NULL)
-		return FALSE;
-
-	/* get results */
-	for (guint i = 0; i < devs->len; i++) {
-		g_autoptr(GError) error_local = NULL;
-		FwupdDevice *dev = g_ptr_array_index (devs, i);
-		if (!fu_util_filter_device (priv, dev))
-			continue;
-		if (!fwupd_client_verify_update (priv->client,
-						 fwupd_device_get_id (dev),
-						 NULL,
-						 &error_local)) {
-			g_print ("%s\tFAILED: %s\n",
-				 fwupd_device_get_guid_default (dev),
-				 error_local->message);
-			continue;
-		}
-		g_print ("%s\t%s\n",
-			 fwupd_device_get_guid_default (dev),
-			 _("OK"));
-	}
-	return TRUE;
-}
-
 static gboolean
 fu_util_verify_update (FuUtilPrivate *priv, gchar **values, GError **error)
 {
 	g_autoptr(FwupdDevice) dev = NULL;
 
-	if (g_strv_length (values) == 0)
-		return fu_util_verify_update_all (priv, error);
-
+	priv->filter_include |= FWUPD_DEVICE_FLAG_CAN_VERIFY;
 	dev = fu_util_get_device_or_prompt (priv, values, error);
 	if (dev == NULL)
 		return FALSE;
 
-	return fwupd_client_verify_update (priv->client, fwupd_device_get_id (dev), NULL, error);
+	if (!fwupd_client_verify_update (priv->client, fwupd_device_get_id (dev), NULL, error)) {
+		g_prefix_error (error, "failed to verify update %s: ", fu_device_get_name (dev));
+		return FALSE;
+	}
+	/* TRANSLATORS: success message when user refreshes device checksums */
+	g_print ("%s\n", _("Successfully updated device checksums"));
+
+	return TRUE;
 }
 
 static gboolean
@@ -1132,7 +1318,10 @@ static gboolean
 fu_util_download_metadata (FuUtilPrivate *priv, GError **error)
 {
 	gboolean download_remote_enabled = FALSE;
+	guint devices_supported_cnt = 0;
+	g_autoptr(GPtrArray) devs = NULL;
 	g_autoptr(GPtrArray) remotes = NULL;
+	g_autoptr(GString) str = g_string_new (NULL);
 
 	remotes = fwupd_client_get_remotes (priv->client, NULL, error);
 	if (remotes == NULL)
@@ -1150,9 +1339,38 @@ fu_util_download_metadata (FuUtilPrivate *priv, GError **error)
 
 	/* no web remote is declared; try to enable LVFS */
 	if (!download_remote_enabled) {
+		/* we don't want to ask anything */
+		if (priv->no_metadata_check) {
+			g_debug ("skipping metadata check");
+			return TRUE;
+		}
+
 		if (!fu_util_download_metadata_enable_lvfs (priv, error))
 			return FALSE;
 	}
+
+	/* get devices from daemon */
+	devs = fwupd_client_get_devices (priv->client, NULL, error);
+	if (devs == NULL)
+		return FALSE;
+
+	/* get results */
+	for (guint i = 0; i < devs->len; i++) {
+		FwupdDevice *dev = g_ptr_array_index (devs, i);
+		if (fwupd_device_has_flag (dev, FWUPD_DEVICE_FLAG_SUPPORTED))
+			devices_supported_cnt++;
+	}
+
+	/* TRANSLATORS: success message -- where 'metadata' is information
+	 * about available firmware on the remote server */
+	g_string_append (str, _("Successfully downloaded new metadata: "));
+
+	/* TRANSLATORS: how many local devices can expect updates now */
+	g_string_append_printf (str, ngettext ("%u local device supported",
+					       "%u local devices supported",
+					       devices_supported_cnt),
+					       devices_supported_cnt);
+	g_print ("%s\n", str->str);
 	return TRUE;
 }
 
@@ -1170,12 +1388,17 @@ fu_util_refresh (FuUtilPrivate *priv, gchar **values, GError **error)
 	}
 
 	/* open file */
-	return fwupd_client_update_metadata (priv->client,
-					     values[2],
-					     values[0],
-					     values[1],
-					     NULL,
-					     error);
+	if (!fwupd_client_update_metadata (priv->client,
+					   values[2],
+					   values[0],
+					   values[1],
+					   NULL,
+					   error))
+		return FALSE;
+
+	/* TRANSLATORS: success message -- the user can do this by-hand too */
+	g_print ("%s\n", _("Successfully refreshed metadata manually"));
+	return TRUE;
 }
 
 static gboolean
@@ -1205,6 +1428,7 @@ fu_util_get_releases (FuUtilPrivate *priv, gchar **values, GError **error)
 	g_autoptr(GNode) root = g_node_new (NULL);
 	g_autofree gchar *title = fu_util_get_tree_title (priv);
 
+	priv->filter_include |= FWUPD_DEVICE_FLAG_SUPPORTED;
 	dev = fu_util_get_device_or_prompt (priv, values, error);
 	if (dev == NULL)
 		return FALSE;
@@ -1285,51 +1509,24 @@ fu_util_prompt_for_release (FuUtilPrivate *priv, GPtrArray *rels, GError **error
 	return g_object_ref (rel);
 }
 
-static gboolean
-fu_util_verify_all (FuUtilPrivate *priv, GError **error)
-{
-	g_autoptr(GPtrArray) devs = NULL;
-
-	/* get devices from daemon */
-	devs = fwupd_client_get_devices (priv->client, NULL, error);
-	if (devs == NULL)
-		return FALSE;
-
-	/* get results */
-	for (guint i = 0; i < devs->len; i++) {
-		g_autoptr(GError) error_local = NULL;
-		FwupdDevice *dev = g_ptr_array_index (devs, i);
-		if (!fu_util_filter_device (priv, dev))
-			continue;
-		if (!fwupd_client_verify (priv->client,
-					  fwupd_device_get_id (dev),
-					  NULL,
-					  &error_local)) {
-			g_print ("%s\tFAILED: %s\n",
-				 fwupd_device_get_guid_default (dev),
-				 error_local->message);
-			continue;
-		}
-		g_print ("%s\t%s\n",
-			 fwupd_device_get_guid_default (dev),
-			 _("OK"));
-	}
-	return TRUE;
-}
-
 static gboolean
 fu_util_verify (FuUtilPrivate *priv, gchar **values, GError **error)
 {
 	g_autoptr(FwupdDevice) dev = NULL;
 
-	if (g_strv_length (values) == 0)
-		return fu_util_verify_all (priv, error);
-
+	priv->filter_include |= FWUPD_DEVICE_FLAG_CAN_VERIFY;
 	dev = fu_util_get_device_or_prompt (priv, values, error);
 	if (dev == NULL)
 		return FALSE;
 
-	return fwupd_client_verify (priv->client, fwupd_device_get_id (dev), NULL, error);
+	if (!fwupd_client_verify (priv->client, fwupd_device_get_id (dev), NULL, error)) {
+		g_prefix_error (error, "failed to verify %s: ", fu_device_get_name (dev));
+		return FALSE;
+	}
+	/* TRANSLATORS: success message when user verified device checksums */
+	g_print ("%s\n", _("Successfully verified device checksums"));
+
+	return TRUE;
 }
 
 static gboolean
@@ -1337,6 +1534,7 @@ fu_util_unlock (FuUtilPrivate *priv, gchar **values, GError **error)
 {
 	g_autoptr(FwupdDevice) dev = NULL;
 
+	priv->filter_include |= FWUPD_DEVICE_FLAG_LOCKED;
 	dev = fu_util_get_device_or_prompt (priv, values, error);
 	if (dev == NULL)
 		return FALSE;
@@ -1423,6 +1621,7 @@ fu_util_get_updates (FuUtilPrivate *priv, gchar **values, GError **error)
 		return FALSE;
 	for (guint i = 0; i < devices->len; i++) {
 		FwupdDevice *dev = g_ptr_array_index (devices, i);
+		g_autofree gchar *upgrade_str = NULL;
 		g_autoptr(GPtrArray) rels = NULL;
 		g_autoptr(GError) error_local = NULL;
 		GNode *child;
@@ -1439,7 +1638,11 @@ fu_util_get_updates (FuUtilPrivate *priv, gchar **values, GError **error)
 						  fwupd_device_get_id (dev),
 						  NULL, &error_local);
 		if (rels == NULL) {
-			g_printerr ("%s\n", error_local->message);
+			/* TRANSLATORS: message letting the user know no device upgrade available
+			* %1 is the device name */
+			upgrade_str = g_strdup_printf (_("No upgrades for %s"),
+						      fwupd_device_get_name (dev));
+			g_printerr ("%s: %s\n", upgrade_str, error_local->message);
 			continue;
 		}
 		child = g_node_append_data (root, dev);
@@ -1510,6 +1713,13 @@ fu_util_update_device_with_release (FuUtilPrivate *priv,
 	g_autofree gchar *uri_str = NULL;
 	g_autoptr(SoupURI) uri = NULL;
 
+	if (!priv->no_safety_check && !priv->assume_yes) {
+		if (!fu_util_prompt_warning (dev,
+					     fu_util_get_tree_title (priv),
+					     error))
+			return FALSE;
+	}
+
 	/* work out what remote-specific URI fields this should use */
 	uri_tmp = fwupd_release_get_uri (rel);
 	remote_id = fwupd_release_get_remote_id (rel);
@@ -1566,10 +1776,38 @@ fu_util_update_device_with_release (FuUtilPrivate *priv,
 				     priv->flags, NULL, error);
 }
 
+static gboolean
+fu_util_maybe_send_reports (FuUtilPrivate *priv, const gchar *remote_id,
+			    GError **error)
+{
+	g_autoptr(FwupdRemote) remote = NULL;
+	g_autoptr(GError) error_local = NULL;
+	if (remote_id == NULL) {
+		g_debug ("not sending reports, no remote");
+		return TRUE;
+	}
+	remote = fwupd_client_get_remote_by_id (priv->client,
+						remote_id,
+						NULL,
+						error);
+	if (remote == NULL)
+		return FALSE;
+	if (fwupd_remote_get_automatic_reports (remote)) {
+		if (!fu_util_report_history (priv, NULL, &error_local))
+			if (!g_error_matches (error_local,
+					      FWUPD_ERROR,
+					      FWUPD_ERROR_NOT_SUPPORTED))
+				g_warning ("%s", error_local->message);
+	}
+
+	return TRUE;
+}
+
 static gboolean
 fu_util_update_all (FuUtilPrivate *priv, GError **error)
 {
 	g_autoptr(GPtrArray) devices = NULL;
+	gboolean supported = FALSE;
 
 	/* get devices from daemon */
 	devices = fwupd_client_get_devices (priv->client, NULL, error);
@@ -1581,6 +1819,8 @@ fu_util_update_all (FuUtilPrivate *priv, GError **error)
 	for (guint i = 0; i < devices->len; i++) {
 		FwupdDevice *dev = g_ptr_array_index (devices, i);
 		FwupdRelease *rel;
+		const gchar *remote_id;
+		g_autofree gchar *upgrade_str = NULL;
 		g_autoptr(GPtrArray) rels = NULL;
 		g_autoptr(GError) error_local = NULL;
 
@@ -1589,20 +1829,46 @@ fu_util_update_all (FuUtilPrivate *priv, GError **error)
 			continue;
 		if (!fu_util_filter_device (priv, dev))
 			continue;
+		supported = TRUE;
 
 		/* get the releases for this device and filter for validity */
 		rels = fwupd_client_get_upgrades (priv->client,
 						  fwupd_device_get_id (dev),
 						  NULL, &error_local);
 		if (rels == NULL) {
-			g_printerr ("%s\n", error_local->message);
+			/* TRANSLATORS: message letting the user know no device upgrade available
+			* %1 is the device name */
+			upgrade_str = g_strdup_printf (_("No upgrades for %s"),
+						      fwupd_device_get_name (dev));
+			g_printerr ("%s: %s\n", upgrade_str, error_local->message);
 			continue;
 		}
 		rel = g_ptr_array_index (rels, 0);
+		/* TRANSLATORS: message letting the user know an upgrade is available
+		 * %1 is the device name and %2 and %3 are version strings */
+		upgrade_str = g_strdup_printf (_("Upgrade available for %s from %s to %s"),
+					       fwupd_device_get_name (dev),
+					       fwupd_device_get_version (dev),
+					       fwupd_release_get_version (rel));
+		g_print ("%s\n", upgrade_str);
 		if (!fu_util_update_device_with_release (priv, dev, rel, error))
 			return FALSE;
 
 		fu_util_display_current_message (priv);
+
+		/* send report if we're supposed to */
+		remote_id = fwupd_release_get_remote_id (rel);
+		if (!fu_util_maybe_send_reports (priv, remote_id, error))
+			return FALSE;
+	}
+
+	/* no devices supported by LVFS or all are filtered */
+	if (!supported) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOTHING_TO_DO,
+				     "No updatable devices");
+		return FALSE;
 	}
 
 	/* we don't want to ask anything */
@@ -1618,6 +1884,7 @@ static gboolean
 fu_util_update_by_id (FuUtilPrivate *priv, const gchar *device_id, GError **error)
 {
 	FwupdRelease *rel;
+	const gchar *remote_id;
 	g_autoptr(FwupdDevice) dev = NULL;
 	g_autoptr(GPtrArray) rels = NULL;
 
@@ -1640,9 +1907,13 @@ fu_util_update_by_id (FuUtilPrivate *priv, const gchar *device_id, GError **erro
 	rel = g_ptr_array_index (rels, 0);
 	if (!fu_util_update_device_with_release (priv, dev, rel, error))
 		return FALSE;
-
 	fu_util_display_current_message (priv);
 
+	/* send report if we're supposed to */
+	remote_id = fwupd_release_get_remote_id (rel);
+	if (!fu_util_maybe_send_reports (priv, remote_id, error))
+		return FALSE;
+
 	/* we don't want to ask anything */
 	if (priv->no_reboot_check) {
 		g_debug ("skipping reboot check");
@@ -1656,6 +1927,22 @@ fu_util_update_by_id (FuUtilPrivate *priv, const gchar *device_id, GError **erro
 static gboolean
 fu_util_update (FuUtilPrivate *priv, gchar **values, GError **error)
 {
+	if (priv->flags & FWUPD_INSTALL_FLAG_ALLOW_OLDER) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_INVALID_ARGS,
+				     "--allow-older is not supported for this command");
+		return FALSE;
+	}
+
+	if (priv->flags & FWUPD_INSTALL_FLAG_ALLOW_REINSTALL) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_INVALID_ARGS,
+				     "--allow-reinstall is not supported for this command");
+		return FALSE;
+	}
+
 	if (g_strv_length (values) == 0)
 		return fu_util_update_all (priv, error);
 	if (g_strv_length (values) == 1)
@@ -1677,7 +1964,12 @@ fu_util_remote_modify (FuUtilPrivate *priv, gchar **values, GError **error)
 				     "Invalid arguments");
 		return FALSE;
 	}
-	return fu_util_modify_remote (priv, values[0], values[1], values[2], error);
+	if (!fu_util_modify_remote (priv, values[0], values[1], values[2], error))
+		return FALSE;
+
+	/* TRANSLATORS: success message for a per-remote setting change */
+	g_print ("%s\n", _("Successfully modified remote"));
+	return TRUE;
 }
 
 static gboolean
@@ -1690,7 +1982,12 @@ fu_util_remote_enable (FuUtilPrivate *priv, gchar **values, GError **error)
 				     "Invalid arguments");
 		return FALSE;
 	}
-	return fu_util_modify_remote (priv, values[0], "Enabled", "true", error);
+	if (!fu_util_modify_remote (priv, values[0], "Enabled", "true", error))
+		return FALSE;
+
+	/* TRANSLATORS: success message */
+	g_print ("%s\n", _("Successfully enabled remote"));
+	return TRUE;
 }
 
 static gboolean
@@ -1703,16 +2000,31 @@ fu_util_remote_disable (FuUtilPrivate *priv, gchar **values, GError **error)
 				     "Invalid arguments");
 		return FALSE;
 	}
-	return fu_util_modify_remote (priv, values[0], "Enabled", "false", error);
+	if (!fu_util_modify_remote (priv, values[0], "Enabled", "false", error))
+		return FALSE;
+
+	/* TRANSLATORS: success message */
+	g_print ("%s\n", _("Successfully disabled remote"));
+	return TRUE;
 }
 
 static gboolean
 fu_util_downgrade (FuUtilPrivate *priv, gchar **values, GError **error)
 {
+	const gchar *remote_id;
 	g_autoptr(FwupdDevice) dev = NULL;
 	g_autoptr(FwupdRelease) rel = NULL;
 	g_autoptr(GPtrArray) rels = NULL;
 
+	if (priv->flags & FWUPD_INSTALL_FLAG_ALLOW_REINSTALL) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_INVALID_ARGS,
+				     "--allow-reinstall is not supported for this command");
+		return FALSE;
+	}
+
+	priv->filter_include |= FWUPD_DEVICE_FLAG_SUPPORTED;
 	dev = fu_util_get_device_or_prompt (priv, values, error);
 	if (dev == NULL)
 		return FALSE;
@@ -1721,8 +2033,14 @@ fu_util_downgrade (FuUtilPrivate *priv, gchar **values, GError **error)
 	rels = fwupd_client_get_downgrades (priv->client,
 					    fwupd_device_get_id (dev),
 					    NULL, error);
-	if (rels == NULL)
+	if (rels == NULL) {
+		/* TRANSLATORS: message letting the user know no device downgrade available
+		 * %1 is the device name */
+		g_autofree gchar *downgrade_str = g_strdup_printf (_("No downgrades for %s"),
+								   fwupd_device_get_name (dev));
+		g_prefix_error (error, "%s: ", downgrade_str);
 		return FALSE;
+	}
 
 	/* get the chosen release */
 	rel = fu_util_prompt_for_release (priv, rels, error);
@@ -1734,7 +2052,78 @@ fu_util_downgrade (FuUtilPrivate *priv, gchar **values, GError **error)
 	g_signal_connect (priv->client, "device-changed",
 			  G_CALLBACK (fu_util_update_device_changed_cb), priv);
 	priv->flags |= FWUPD_INSTALL_FLAG_ALLOW_OLDER;
-	return fu_util_update_device_with_release (priv, dev, rel, error);
+	if (!fu_util_update_device_with_release (priv, dev, rel, error))
+		return FALSE;
+
+	/* TRANSLATORS: success message where we made the firmware on the
+	 * device older than it was before */
+	g_print ("%s\n", _("Successfully downgraded device"));
+
+	/* send report if we're supposed to */
+	remote_id = fwupd_release_get_remote_id (rel);
+	if (!fu_util_maybe_send_reports (priv, remote_id, error))
+		return FALSE;
+
+	return TRUE;
+}
+
+static gboolean
+fu_util_reinstall (FuUtilPrivate *priv, gchar **values, GError **error)
+{
+	const gchar *remote_id;
+	g_autoptr(FwupdRelease) rel = NULL;
+	g_autoptr(GPtrArray) rels = NULL;
+	g_autoptr(FwupdDevice) dev = NULL;
+
+	priv->filter_include |= FWUPD_DEVICE_FLAG_SUPPORTED;
+	dev = fu_util_get_device_or_prompt (priv, values, error);
+	if (dev == NULL)
+		return FALSE;
+
+	/* try to lookup/match release from client */
+	rels = fwupd_client_get_releases (priv->client, fwupd_device_get_id (dev),
+					  NULL, error);
+	if (rels == NULL)
+		return FALSE;
+	for (guint j = 0; j < rels->len; j++) {
+		FwupdRelease *rel_tmp = g_ptr_array_index (rels, j);
+		if (fu_common_vercmp (fwupd_release_get_version (rel_tmp),
+		    fu_device_get_version (dev)) == 0) {
+			rel = g_object_ref (rel_tmp);
+			break;
+		}
+	}
+	if (rel == NULL) {
+		g_set_error (error,
+			     FWUPD_ERROR,
+			     FWUPD_ERROR_NOT_SUPPORTED,
+			     "Unable to locate release for %s version %s",
+			     fu_device_get_name (dev),
+			     fu_device_get_version (dev));
+		return FALSE;
+	}
+
+	/* update the console if composite devices are also updated */
+	priv->current_operation = FU_UTIL_OPERATION_INSTALL;
+	g_signal_connect (priv->client, "device-changed",
+			  G_CALLBACK (fu_util_update_device_changed_cb), priv);
+	priv->flags |= FWUPD_INSTALL_FLAG_ALLOW_REINSTALL;
+	if (!fu_util_update_device_with_release (priv, dev, rel, error))
+		return FALSE;
+	fu_util_display_current_message (priv);
+
+	/* send report if we're supposed to */
+	remote_id = fwupd_release_get_remote_id (rel);
+	if (!fu_util_maybe_send_reports (priv, remote_id, error))
+		return FALSE;
+
+	/* we don't want to ask anything */
+	if (priv->no_reboot_check) {
+		g_debug ("skipping reboot check");
+		return TRUE;
+	}
+
+	return fu_util_prompt_complete (priv->completion_flags, TRUE, error);
 }
 
 static gboolean
@@ -1799,6 +2188,9 @@ fu_util_activate (FuUtilPrivate *priv, gchar **values, GError **error)
 			return FALSE;
 	}
 
+	/* TRANSLATORS: success message -- where activation is making the new
+	 * firmware take effect, usually after updating offline */
+	g_print ("%s\n", _("Successfully activated all devices"));
 	return TRUE;
 }
 
@@ -1877,7 +2269,7 @@ fu_util_modify_config (FuUtilPrivate *priv, gchar **values, GError **error)
 					 error))
 		return FALSE;
 	if (!priv->assume_yes) {
-		g_print ("%s\n [Y|n]: ",
+		g_print ("%s [Y|n]: ",
 			/* TRANSLATORS: configuration changes only take effect on restart */
 			 _("Restart the daemon to make the change effective?"));
 		if (!fu_util_prompt_for_boolean (FALSE))
@@ -1887,6 +2279,8 @@ fu_util_modify_config (FuUtilPrivate *priv, gchar **values, GError **error)
 	if (!fu_systemd_unit_stop (fu_util_get_systemd_unit (), error))
 		return FALSE;
 #endif
+	/* TRANSLATORS: success message -- a per-system setting value */
+	g_print ("%s\n", _("Successfully modified configuration value"));
 	return TRUE;
 }
 
@@ -1896,6 +2290,7 @@ fu_util_ignore_cb (const gchar *log_domain, GLogLevelFlags log_level,
 {
 }
 
+#ifdef HAVE_GIO_UNIX
 static gboolean
 fu_util_sigint_cb (gpointer user_data)
 {
@@ -1904,6 +2299,7 @@ fu_util_sigint_cb (gpointer user_data)
 	g_cancellable_cancel (priv->cancellable);
 	return FALSE;
 }
+#endif
 
 static void
 fu_util_private_free (FuUtilPrivate *priv)
@@ -1925,10 +2321,7 @@ fu_util_private_free (FuUtilPrivate *priv)
 static gboolean
 fu_util_check_daemon_version (FuUtilPrivate *priv, GError **error)
 {
-	g_autofree gchar *client = g_strdup_printf ("%i.%i.%i",
-						    FWUPD_MAJOR_VERSION,
-						    FWUPD_MINOR_VERSION,
-						    FWUPD_MICRO_VERSION);
+	g_autofree gchar *client = fu_util_get_client_version ();
 	const gchar *daemon = fwupd_client_get_daemon_version (priv->client);
 
 	if (g_strcmp0 (daemon, client) != 0) {
@@ -2031,7 +2424,7 @@ main (int argc, char *argv[])
 			_("Schedule installation for next reboot when possible"), NULL },
 		{ "allow-reinstall", '\0', 0, G_OPTION_ARG_NONE, &allow_reinstall,
 			/* TRANSLATORS: command line option */
-			_("Allow re-installing existing firmware versions"), NULL },
+			_("Allow reinstalling existing firmware versions"), NULL },
 		{ "allow-older", '\0', 0, G_OPTION_ARG_NONE, &allow_older,
 			/* TRANSLATORS: command line option */
 			_("Allow downgrading firmware versions"), NULL },
@@ -2056,6 +2449,9 @@ main (int argc, char *argv[])
 		{ "no-reboot-check", '\0', 0, G_OPTION_ARG_NONE, &priv->no_reboot_check,
 			/* TRANSLATORS: command line option */
 			_("Do not check for reboot after update"), NULL },
+		{ "no-safety-check", '\0', 0, G_OPTION_ARG_NONE, &priv->no_safety_check,
+			/* TRANSLATORS: command line option */
+			_("Do not perform device safety checks"), NULL },
 		{ "no-history", '\0', 0, G_OPTION_ARG_NONE, &no_history,
 			/* TRANSLATORS: command line option */
 			_("Do not write to the history database"), NULL },
@@ -2074,7 +2470,7 @@ main (int argc, char *argv[])
 
 	setlocale (LC_ALL, "");
 
-	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	bindtextdomain (GETTEXT_PACKAGE, FWUPD_LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
@@ -2138,7 +2534,7 @@ main (int argc, char *argv[])
 		     "verify",
 		     "[DEVICE_ID]",
 		     /* TRANSLATORS: command description */
-		     _("Gets the cryptographic hash of the dumped firmware"),
+		     _("Checks cryptographic hash matches firmware"),
 		     fu_util_verify);
 	fu_util_cmd_array_add (cmd_array,
 		     "unlock",
@@ -2192,7 +2588,7 @@ main (int argc, char *argv[])
 		     "verify-update",
 		     "[DEVICE_ID]",
 		     /* TRANSLATORS: command description */
-		     _("Update the stored metadata with current ROM contents"),
+		     _("Update the stored cryptographic hash with current ROM contents"),
 		     fu_util_verify_update);
 	fu_util_cmd_array_add (cmd_array,
 		     "modify-remote",
@@ -2236,12 +2632,21 @@ main (int argc, char *argv[])
 		     /* TRANSLATORS: sets something in daemon.conf */
 		     _("Modifies a daemon configuration value."),
 		     fu_util_modify_config);
+	fu_util_cmd_array_add (cmd_array,
+		     "reinstall",
+		     "[DEVICE-ID]",
+		     /* TRANSLATORS: command description */
+		     _("Reinstall current firmware on the device."),
+		     fu_util_reinstall);
+
 
 	/* do stuff on ctrl+c */
 	priv->cancellable = g_cancellable_new ();
+#ifdef HAVE_GIO_UNIX
 	g_unix_signal_add_full (G_PRIORITY_DEFAULT,
 				SIGINT, fu_util_sigint_cb,
 				priv, NULL);
+#endif
 
 	/* sort by command name */
 	fu_util_cmd_array_sort (cmd_array);
@@ -2281,6 +2686,7 @@ main (int argc, char *argv[])
 		priv->no_unreported_check = TRUE;
 		priv->no_metadata_check = TRUE;
 		priv->no_reboot_check = TRUE;
+		priv->no_safety_check = TRUE;
 		fu_progressbar_set_interactive (priv->progressbar, FALSE);
 	}
 
@@ -2360,6 +2766,7 @@ main (int argc, char *argv[])
 #ifdef HAVE_SYSTEMD
 	/* make sure the correct daemon is in use */
 	if ((priv->flags & FWUPD_INSTALL_FLAG_FORCE) == 0 &&
+	    !fwupd_client_get_daemon_interactive (priv->client) &&
 	    !fu_util_using_correct_daemon (&error)) {
 		g_printerr ("%s\n", error->message);
 		return EXIT_FAILURE;
